
#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# pylint:disable=bad-whitespace
# pylint:disable=line-too-long
# pylint:disable=too-many-lines
# pylint:disable=invalid-name

# #########################################################
#
#       ************** !! WARNING !! ***************
#       ******* THIS FILE WAS AUTO-GENERATED *******
#       ********* DO NOT MODIFY THIS FILE **********
#
# #########################################################

wordnet_terms_h_provenance = {
    'action': ['router.py',
               'plac_core.py',
               'csv2py_orchestrator.py',
               'wordnet2py.py',
               'csv_list_writer.py',
               'common_utils.py'],
    'source': 'resources/data/csv/wordnet.csv',
    'time': '2022-05-17 17:00:36.166458'}

wordnet_terms_h = [
    'ha',
    'haart',
    'haastia',
    'habacuc',
    'habakkuk',
    'habanera',
    'habeas',
    'habenaria',
    'haber',
    'haberdasher',
    'haberdashery',
    'habergeon',
    'habiliment',
    'habilimented',
    'habilitate',
    'habilitated',
    'habit',
    'habitability',
    'habitable',
    'habitableness',
    'habitant',
    'habitat',
    'habitation',
    'habitations',
    'habitats',
    'habited',
    'habits',
    'habitual',
    'habitually',
    'habituate',
    'habituated',
    'habituation',
    'habitude',
    'habitue',
    'habitus',
    'habsburg',
    'habsburgs',
    'hacek',
    'hachiman',
    'hachure',
    'hacienda',
    'hack',
    'hackamore',
    'hackberry',
    'hackbut',
    'hacked',
    'hackee',
    'hackelia',
    'hacker',
    'hackers',
    'hacking',
    'hackle',
    'hackles',
    'hackmatack',
    'hackney',
    'hackneyed',
    'hacks',
    'hacksaw',
    'hackwork',
    'had',
    'hadal',
    'haddock',
    'hadean',
    'hades',
    'hadith',
    'hadj',
    'hadji',
    'hadrian',
    'hadron',
    'hadrosaur',
    'hadrosauridae',
    'hadrosaurus',
    'haecceity',
    'haeckel',
    'haem',
    'haemagglutinate',
    'haemagglutination',
    'haemal',
    'haemangioma',
    'haemanthus',
    'haematal',
    'haematemesis',
    'haematic',
    'haematinic',
    'haematite',
    'haematobia',
    'haematocele',
    'haematochezia',
    'haematocoele',
    'haematocolpometra',
    'haematocolpos',
    'haematocrit',
    'haematocytopenia',
    'haematocyturia',
    'haematogenesis',
    'haematogenic',
    'haematohiston',
    'haematoidin',
    'haematological',
    'haematologist',
    'haematology',
    'haematolysis',
    'haematoma',
    'haematopodidae',
    'haematopoiesis',
    'haematopoietic',
    'haematopus',
    'haematoxylon',
    'haematoxylum',
    'haematuria',
    'haemic',
    'haemitin',
    'haemodialysis',
    'haemodoraceae',
    'haemodorum',
    'haemogenesis',
    'haemoglobin',
    'haemoglobinemia',
    'haemoglobinopathy',
    'haemoglobinuria',
    'haemolysin',
    'haemolysis',
    'haemolytic',
    'haemophile',
    'haemophilia',
    'haemophiliac',
    'haemophilic',
    'haemopis',
    'haemopoiesis',
    'haemopoietic',
    'haemoproteid',
    'haemoproteidae',
    'haemoprotein',
    'haemoproteus',
    'haemoptysis',
    'haemorrhage',
    'haemorrhagic',
    'haemorrhoid',
    'haemorrhoidectomy',
    'haemosiderin',
    'haemosiderosis',
    'haemosporidia',
    'haemosporidian',
    'haemostasia',
    'haemostasis',
    'haemostat',
    'haemothorax',
    'haemulidae',
    'haemulon',
    'hafnium',
    'haft',
    'haftarah',
    'haftorah',
    'hag',
    'hagada',
    'haganah',
    'hagberry',
    'hagbut',
    'hagerstown',
    'hagfish',
    'hagfishes',
    'haggada',
    'haggadah',
    'haggai',
    'haggard',
    'haggis',
    'haggle',
    'haggler',
    'haggling',
    'hagiographa',
    'hagiographer',
    'hagiographist',
    'hagiography',
    'hagiolatry',
    'hagiologist',
    'hagiology',
    'hagridden',
    'hahn',
    'hahnium',
    'hai',
    'haick',
    'haida',
    'haifa',
    'haik',
    'haiku',
    'hail',
    'hailed',
    'hails',
    'hailstone',
    'hailstorm',
    'haiphong',
    'hair',
    'hairball',
    'hairbrush',
    'haircare',
    'haircloth',
    'haircut',
    'haircuts',
    'hairdo',
    'hairdresser',
    'hairdressers',
    'hairdressing',
    'haired',
    'hairgrip',
    'hairiness',
    'hairless',
    'hairlessness',
    'hairlike',
    'hairline',
    'hairnet',
    'hairpiece',
    'hairpin',
    'hairs',
    'hairsbreadth',
    'hairsplitter',
    'hairsplitting',
    'hairspring',
    'hairstreak',
    'hairstyle',
    'hairstylist',
    'hairtail',
    'hairweaving',
    'hairy',
    'haiti',
    'haitian',
    'haj',
    'haji',
    'hajj',
    'hajji',
    'hake',
    'hakea',
    'hakeem',
    'hakenkreuz',
    'hakes',
    'hakham',
    'hakim',
    'hakka',
    'hal',
    'halab',
    'halacha',
    'halaka',
    'halakah',
    'halal',
    'halberd',
    'halberdier',
    'halchidhoma',
    'halcion',
    'halcyon',
    'haldane',
    'haldea',
    'haldol',
    'hale',
    'haleness',
    'halenia',
    'haler',
    'halers',
    'halesia',
    'halevy',
    'haley',
    'half',
    'halfback',
    'halfbeak',
    'halfbeaks',
    'halfhearted',
    'halfpenny',
    'halfpennyworth',
    'halftime',
    'halftone',
    'halfway',
    'haliaeetus',
    'halibut',
    'halibuts',
    'halicarnassus',
    'halicoeres',
    'halictidae',
    'halide',
    'halides',
    'halifax',
    'halimodendron',
    'haliotidae',
    'haliotis',
    'halite',
    'halitosis',
    'halitus',
    'hall',
    'hallah',
    'halle',
    'hallel',
    'hallelujah',
    'halley',
    'halliard',
    'hallmark',
    'halloo',
    'hallow',
    'hallowed',
    'halloween',
    'hallowmas',
    'hallowmass',
    'halls',
    'hallstand',
    'hallucinate',
    'hallucinated',
    'hallucinating',
    'hallucination',
    'hallucinations',
    'hallucinatory',
    'hallucinogen',
    'hallucinogenic',
    'hallucinosis',
    'hallux',
    'hallway',
    'hallways',
    'halm',
    'halma',
    'halo',
    'haloalkane',
    'halobacter',
    'halobacteria',
    'halobacterium',
    'halocarbon',
    'halocarpus',
    'haloform',
    'halogen',
    'halogens',
    'halogeton',
    'halon',
    'haloperidol',
    'halophil',
    'halophile',
    'halophiles',
    'halophyte',
    'halophytes',
    'haloragaceae',
    'haloragidaceae',
    'halothane',
    'hals',
    'halt',
    'halter',
    'haltere',
    'halting',
    'halts',
    'halve',
    'halved',
    'halves',
    'halyard',
    'ham',
    'hamadryad',
    'hamamelidaceae',
    'hamamelidae',
    'hamamelidanthum',
    'hamamelidoxylon',
    'hamamelis',
    'hamamelites',
    'haman',
    'hamartia',
    'hamartoma',
    'hamas',
    'hamate',
    'hamburg',
    'hamburger',
    'hamburgers',
    'hame',
    'hamelia',
    'hamelin',
    'hameln',
    'hamilton',
    'haminoea',
    'hamitic',
    'hamlet',
    'hammadi',
    'hammarskjold',
    'hammer',
    'hammered',
    'hammerhead',
    'hammering',
    'hammerlock',
    'hammers',
    'hammerstein',
    'hammertoe',
    'hammett',
    'hamming',
    'hammock',
    'hammocks',
    'hammurabi',
    'hammurapi',
    'hammy',
    'hamper',
    'hampered',
    'hampshire',
    'hampton',
    'hamster',
    'hamsters',
    'hamstring',
    'hamstrings',
    'hamstrung',
    'hamsun',
    'han',
    'hancock',
    'hand',
    'handbag',
    'handball',
    'handbarrow',
    'handbasin',
    'handbasket',
    'handbell',
    'handbells',
    'handbill',
    'handbook',
    'handbow',
    'handbreadth',
    'handbuild',
    'handcar',
    'handcart',
    'handclap',
    'handclasp',
    'handcolor',
    'handcolour',
    'handcraft',
    'handcrafts',
    'handcuff',
    'handcuffed',
    'handcuffs',
    'handed',
    'handedness',
    'handel',
    'handelian',
    'handful',
    'handgrip',
    'handgun',
    'handheld',
    'handhold',
    'handicap',
    'handicapped',
    'handicapper',
    'handicraft',
    'handicrafts',
    'handiness',
    'handing',
    'handiwork',
    'handkerchief',
    'handle',
    'handlebar',
    'handlebars',
    'handled',
    'handleless',
    'handler',
    'handles',
    'handless',
    'handline',
    'handling',
    'handlock',
    'handloom',
    'handmade',
    'handmaid',
    'handmaiden',
    'handoff',
    'handout',
    'handover',
    'handrail',
    'handrails',
    'handrest',
    'hands',
    'handsaw',
    'handsaws',
    'handsbreadth',
    'handset',
    'handsewn',
    'handshake',
    'handshaking',
    'handsome',
    'handsomely',
    'handsomeness',
    'handspike',
    'handspring',
    'handstamp',
    'handstand',
    'handstands',
    'handstitched',
    'handwash',
    'handwear',
    'handwheel',
    'handwork',
    'handwoven',
    'handwrite',
    'handwriting',
    'handwritten',
    'handy',
    'handyman',
    'hang',
    'hangar',
    'hangars',
    'hangbird',
    'hangchow',
    'hangdog',
    'hanged',
    'hanger',
    'hanging',
    'hangman',
    'hangnail',
    'hangout',
    'hangover',
    'hangs',
    'hangzhou',
    'hani',
    'hank',
    'hanker',
    'hankering',
    'hankey',
    'hankie',
    'hanks',
    'hanky',
    'hannibal',
    'hannover',
    'hannukah',
    'hanoi',
    'hanover',
    'hanoverian',
    'hans',
    'hansard',
    'hanseatic',
    'hansom',
    'hanukah',
    'hanukkah',
    'hanuman',
    'hao',
    'haoma',
    'hap',
    'haphazard',
    'haphazardly',
    'haphazardness',
    'haphtarah',
    'haphtorah',
    'hapless',
    'haploid',
    'haploidic',
    'haploidy',
    'haplopappus',
    'haplosporidia',
    'haplosporidian',
    'haplotype',
    'happen',
    'happend',
    'happened',
    'happening',
    'happens',
    'happenstance',
    'happier',
    'happiest',
    'happiness',
    'happy',
    'haps',
    'hapsburg',
    'hapsburgs',
    'haptic',
    'haptoglobin',
    'harakiri',
    'harangue',
    'haranguer',
    'harare',
    'harass',
    'harassed',
    'harasser',
    'harassers',
    'harasses',
    'harassing',
    'harassment',
    'harbinger',
    'harbingers',
    'harbor',
    'harborage',
    'harboring',
    'harbors',
    'harbour',
    'harbourage',
    'hard',
    'hardback',
    'hardbacked',
    'hardbake',
    'hardball',
    'hardboard',
    'hardbound',
    'hardcore',
    'hardcover',
    'harden',
    'hardenbergia',
    'hardened',
    'hardening',
    'hardens',
    'harder',
    'hardest',
    'hardfisted',
    'hardheaded',
    'hardheads',
    'hardhearted',
    'hardheartedness',
    'hardiest',
    'hardihood',
    'hardiness',
    'harding',
    'hardinggrass',
    'hardline',
    'hardliner',
    'hardliners',
    'hardly',
    'hardness',
    'hardpan',
    'hardscrabble',
    'hardship',
    'hardships',
    'hardtack',
    'hardtop',
    'hardware',
    'hardwareman',
    'hardwood',
    'hardwoods',
    'hardworking',
    'hardy',
    'hare',
    'harebell',
    'harebrained',
    'hared',
    'haredi',
    'hareem',
    'harefoot',
    'harelip',
    'harem',
    'hares',
    'hargeisa',
    'hargreaves',
    'haricot',
    'harijan',
    'harikari',
    'hark',
    'harken',
    'harks',
    'harlem',
    'harlequin',
    'harlequinade',
    'harlequined',
    'harlot',
    'harlotry',
    'harlow',
    'harm',
    'harmattan',
    'harmed',
    'harmful',
    'harmfulness',
    'harming',
    'harmless',
    'harmonic',
    'harmonica',
    'harmonical',
    'harmonically',
    'harmonics',
    'harmonies',
    'harmonious',
    'harmoniously',
    'harmoniousness',
    'harmonisation',
    'harmonise',
    'harmonised',
    'harmoniser',
    'harmonium',
    'harmonizable',
    'harmonization',
    'harmonize',
    'harmonized',
    'harmonizer',
    'harmonizes',
    'harmonizing',
    'harmony',
    'harms',
    'harmsworth',
    'harness',
    'harnessed',
    'harold',
    'harp',
    'harped',
    'harper',
    'harpers',
    'harpia',
    'harping',
    'harpist',
    'harpo',
    'harpoon',
    'harpooned',
    'harpooneer',
    'harpooner',
    'harpoons',
    'harpsichord',
    'harpsichordist',
    'harpulla',
    'harpullia',
    'harpy',
    'harquebus',
    'harridan',
    'harried',
    'harrier',
    'harriers',
    'harriet',
    'harriman',
    'harris',
    'harrisburg',
    'harrisia',
    'harrison',
    'harrod',
    'harrow',
    'harrowed',
    'harrowing',
    'harry',
    'harsh',
    'harshen',
    'harshened',
    'harshing',
    'harshly',
    'harshness',
    'hart',
    'harte',
    'hartebeest',
    'hartebeests',
    'hartford',
    'hartley',
    'harvard',
    'harvest',
    'harvested',
    'harvester',
    'harvesters',
    'harvestfish',
    'harvesting',
    'harvestman',
    'harvestmen',
    'harvests',
    'harvey',
    'has',
    'haschisch',
    'hasdrubal',
    'hasek',
    'hash',
    'hasheesh',
    'hashish',
    'hashmark',
    'hasid',
    'hasidic',
    'hasidim',
    'hasidism',
    'haslet',
    'hasp',
    'hassam',
    'hassel',
    'hassid',
    'hassidic',
    'hassidim',
    'hassidism',
    'hassium',
    'hassle',
    'hassock',
    'hastate',
    'haste',
    'hasten',
    'hastily',
    'hastinapura',
    'hastiness',
    'hastings',
    'hasty',
    'hat',
    'hatband',
    'hatbox',
    'hatch',
    'hatchback',
    'hatched',
    'hatchel',
    'hatchery',
    'hatchet',
    'hatching',
    'hatchling',
    'hatchlings',
    'hatchway',
    'hate',
    'hated',
    'hateful',
    'hatefulness',
    'hatemonger',
    'hater',
    'hates',
    'hatful',
    'hath',
    'hathaway',
    'hating',
    'hatiora',
    'hatless',
    'hatmaker',
    'hatpin',
    'hatrack',
    'hatred',
    'hats',
    'hatted',
    'hatter',
    'hatteras',
    'hattiesburg',
    'hauberk',
    'haughtily',
    'haughtiness',
    'haughty',
    'haul',
    'haulage',
    'hauled',
    'hauler',
    'haulier',
    'hauling',
    'haulm',
    'haunch',
    'haunt',
    'haunted',
    'haunting',
    'haunts',
    'hausa',
    'hausen',
    'hausmannite',
    'haussa',
    'haustoria',
    'haustorium',
    'hautbois',
    'hautboy',
    'haute',
    'hauteur',
    'havana',
    'havasupai',
    'have',
    'haveing',
    'havel',
    'havelock',
    'haven',
    'haversack',
    'having',
    'havoc',
    'haw',
    'hawaii',
    'hawaiian',
    'hawala',
    'hawed',
    'hawfinch',
    'hawk',
    'hawkbill',
    'hawkbit',
    'hawker',
    'hawking',
    'hawkins',
    'hawkish',
    'hawkishness',
    'hawkmoth',
    'hawkmoths',
    'hawks',
    'hawksbill',
    'hawksbills',
    'hawkshaw',
    'hawkweed',
    'hawkyns',
    'haworth',
    'haws',
    'hawse',
    'hawsehole',
    'hawsepipe',
    'hawser',
    'hawsers',
    'hawthorn',
    'hawthorne',
    'hay',
    'hayastan',
    'haycock',
    'haydn',
    'hayek',
    'hayes',
    'hayfield',
    'hayfork',
    'haying',
    'hayloft',
    'haymaker',
    'haymaking',
    'haymow',
    'hayrack',
    'hayrick',
    'hayrig',
    'hays',
    'hayseed',
    'haystack',
    'hayti',
    'haywire',
    'haywood',
    'hazan',
    'hazard',
    'hazardia',
    'hazardous',
    'hazardousness',
    'hazards',
    'haze',
    'hazel',
    'hazelnut',
    'hazelnuts',
    'hazelwood',
    'haziness',
    'hazlitt',
    'hazmat',
    'hazmats',
    'hazy',
    'hb',
    'hcfc',
    'hcg',
    'hcooh',
    'hdl',
    'hdtv',
    'he',
    'head',
    'headache',
    'headaches',
    'headband',
    'headboard',
    'headbutt',
    'headbutted',
    'headcheese',
    'headcount',
    'headcounter',
    'headdress',
    'headed',
    'header',
    'headfast',
    'headfirst',
    'headfish',
    'headful',
    'headgear',
    'headhunter',
    'heading',
    'headings',
    'headlamp',
    'headland',
    'headless',
    'headlight',
    'headlights',
    'headlike',
    'headline',
    'headliner',
    'headlines',
    'headlinese',
    'headlock',
    'headlong',
    'headman',
    'headmaster',
    'headmastership',
    'headmistress',
    'headmistressship',
    'headphone',
    'headphones',
    'headpiece',
    'headpin',
    'headquarter',
    'headquartered',
    'headquarters',
    'headrace',
    'headrest',
    'headroom',
    'heads',
    'headsail',
    'headscarf',
    'headset',
    'headshake',
    'headshakes',
    'headshaking',
    'headship',
    'headshot',
    'headsman',
    'headspace',
    'headspring',
    'headstall',
    'headstand',
    'headstock',
    'headstone',
    'headstones',
    'headstream',
    'headstrong',
    'headwaiter',
    'headwater',
    'headwaters',
    'headway',
    'headwind',
    'headword',
    'heady',
    'heal',
    'healed',
    'healer',
    'healing',
    'heals',
    'health',
    'healthcare',
    'healthful',
    'healthfulness',
    'healthier',
    'healthily',
    'healthiness',
    'healthy',
    'heap',
    'heaped',
    'heaps',
    'hear',
    'hearable',
    'heard',
    'hearer',
    'hearers',
    'hearing',
    'hearings',
    'hearken',
    'hears',
    'hearsay',
    'hearse',
    'hearst',
    'heart',
    'heartache',
    'heartbeat',
    'heartbreak',
    'heartbreaker',
    'heartbreaking',
    'heartbroken',
    'heartburn',
    'heartburning',
    'hearten',
    'heartened',
    'heartening',
    'heartfelt',
    'hearth',
    'hearthrug',
    'hearths',
    'hearthstone',
    'heartily',
    'heartiness',
    'heartland',
    'heartleaf',
    'heartless',
    'heartlessness',
    'heartrending',
    'heartrot',
    'hearts',
    'heartsease',
    'heartseed',
    'heartsick',
    'heartsickness',
    'heartstrings',
    'heartthrob',
    'heartwarming',
    'heartwood',
    'hearty',
    'heat',
    'heatable',
    'heated',
    'heater',
    'heaters',
    'heath',
    'heathen',
    'heathenish',
    'heathenism',
    'heather',
    'heathers',
    'heathfowl',
    'heathland',
    'heathlike',
    'heaths',
    'heating',
    'heatless',
    'heats',
    'heatstroke',
    'heaume',
    'heave',
    'heaved',
    'heaven',
    'heavenly',
    'heavens',
    'heavenward',
    'heaver',
    'heaves',
    'heavier',
    'heaviest',
    'heavily',
    'heaviness',
    'heaving',
    'heaviside',
    'heavy',
    'heavyhearted',
    'heavyheartedness',
    'heavyset',
    'heavyweight',
    'hebbel',
    'hebdomad',
    'hebdomadal',
    'hebdomadary',
    'hebe',
    'hebei',
    'hebephrenia',
    'hebephrenic',
    'hebetude',
    'hebraic',
    'hebraical',
    'hebraism',
    'hebraist',
    'hebrew',
    'hebrews',
    'hebridean',
    'hebrides',
    'hecate',
    'hecatomb',
    'hecht',
    'heckelphone',
    'heckle',
    'heckler',
    'hecklers',
    'heckling',
    'hectare',
    'hectic',
    'hectogram',
    'hectograph',
    'hectoliter',
    'hectolitre',
    'hectometer',
    'hectometre',
    'hector',
    'hectoring',
    'hedeoma',
    'hedera',
    'hedge',
    'hedged',
    'hedgefund',
    'hedgehog',
    'hedgehogs',
    'hedgehop',
    'hedger',
    'hedgerow',
    'hedgerows',
    'hedges',
    'hedging',
    'hediondilla',
    'hedjaz',
    'hedonic',
    'hedonism',
    'hedonist',
    'hedonistic',
    'hedysarum',
    'heed',
    'heedful',
    'heedfulness',
    'heedless',
    'heedlessness',
    'heel',
    'heelbone',
    'heeled',
    'heels',
    'hefa',
    'heft',
    'heftiness',
    'hefty',
    'hegari',
    'hegel',
    'hegelian',
    'hegemon',
    'hegemony',
    'hegira',
    'heidegger',
    'heifer',
    'heifers',
    'height',
    'heighten',
    'heightened',
    'heightening',
    'heightens',
    'heights',
    'heilong',
    'heimdal',
    'heimdall',
    'heimdallr',
    'heinlein',
    'heinous',
    'heinousness',
    'heinrich',
    'heinz',
    'heir',
    'heiress',
    'heirloom',
    'heirs',
    'heisenberg',
    'heist',
    'hejaz',
    'hejira',
    'hel',
    'hela',
    'held',
    'helen',
    'helena',
    'helenium',
    'heleodytes',
    'heliac',
    'heliacal',
    'heliamphora',
    'helianthemum',
    'helianthemums',
    'helianthus',
    'helical',
    'helices',
    'helichrysum',
    'helicidae',
    'helicon',
    'helicopter',
    'helicopters',
    'helicteres',
    'heliobacter',
    'heliocentric',
    'heliogram',
    'heliograph',
    'heliogravure',
    'heliolatry',
    'heliometer',
    'heliopause',
    'heliophila',
    'heliopsis',
    'helios',
    'heliosphere',
    'heliotherapy',
    'heliothis',
    'heliotrope',
    'heliotropism',
    'heliotype',
    'heliozoa',
    'heliozoan',
    'heliport',
    'helipterum',
    'helium',
    'helix',
    'hell',
    'hellbender',
    'hellcat',
    'hellebore',
    'helleborine',
    'helleborus',
    'hellene',
    'hellenic',
    'hellenism',
    'hellenistic',
    'hellenistical',
    'heller',
    'helleri',
    'hellespont',
    'hellfire',
    'hellgrammiate',
    'hellhole',
    'hellhound',
    'hellion',
    'hellions',
    'hellish',
    'hellman',
    'hello',
    'helm',
    'helmet',
    'helmeted',
    'helmetflower',
    'helmets',
    'helmholtz',
    'helminth',
    'helminthiasis',
    'helminthic',
    'helminthostachys',
    'helms',
    'helmsman',
    'heloderma',
    'helodermatidae',
    'heloise',
    'helot',
    'helotiaceae',
    'helotiales',
    'helotium',
    'help',
    'helpdesk',
    'helped',
    'helper',
    'helpful',
    'helpfulness',
    'helping',
    'helpings',
    'helpless',
    'helplessly',
    'helplessness',
    'helpmate',
    'helpmeet',
    'helps',
    'helsingfors',
    'helsinki',
    'helve',
    'helvella',
    'helvellaceae',
    'helvetia',
    'helvetian',
    'helvetica',
    'helwingia',
    'helxine',
    'hem',
    'hemachatus',
    'hemagglutinate',
    'hemagglutination',
    'hemal',
    'hemangioma',
    'hematal',
    'hematemesis',
    'hematic',
    'hematin',
    'hematinic',
    'hematite',
    'hematocele',
    'hematochezia',
    'hematochrome',
    'hematocoele',
    'hematocolpometra',
    'hematocolpos',
    'hematocrit',
    'hematocyst',
    'hematocytopenia',
    'hematocyturia',
    'hematogenesis',
    'hematogenic',
    'hematohiston',
    'hematoidin',
    'hematologic',
    'hematological',
    'hematologist',
    'hematology',
    'hematolysis',
    'hematoma',
    'hematopoiesis',
    'hematopoietic',
    'hematuria',
    'heme',
    'hemeralopia',
    'hemerobiid',
    'hemerobiidae',
    'hemerocallidaceae',
    'hemerocallis',
    'hemiacetal',
    'hemianopia',
    'hemianopsia',
    'hemiascomycetes',
    'hemic',
    'hemicrania',
    'hemicycle',
    'hemidemisemiquaver',
    'hemiepiphyte',
    'hemigalus',
    'hemigrammus',
    'hemimetabola',
    'hemimetabolic',
    'hemimetabolism',
    'hemimetabolous',
    'hemimetaboly',
    'hemimetamorphic',
    'hemimetamorphosis',
    'hemimetamorphous',
    'hemimorphite',
    'hemin',
    'heming',
    'hemingway',
    'hemingwayesque',
    'hemiparasite',
    'hemiparasites',
    'hemiparasitic',
    'hemiplegia',
    'hemiplegic',
    'hemipode',
    'hemiprocnidae',
    'hemiptera',
    'hemipteran',
    'hemipteron',
    'hemipteronatus',
    'hemiramphidae',
    'hemisphere',
    'hemispheres',
    'hemispheric',
    'hemispherical',
    'hemitripterus',
    'hemline',
    'hemlock',
    'hemlocks',
    'hemmed',
    'hemming',
    'hemminge',
    'hemochromatosis',
    'hemodialysis',
    'hemodialyzer',
    'hemodynamic',
    'hemodynamics',
    'hemofil',
    'hemogenesis',
    'hemoglobin',
    'hemoglobinemia',
    'hemoglobinopathy',
    'hemoglobins',
    'hemoglobinuria',
    'hemolysin',
    'hemolysins',
    'hemolysis',
    'hemolytic',
    'hemophile',
    'hemophilia',
    'hemophiliac',
    'hemophilic',
    'hemopoiesis',
    'hemopoietic',
    'hemoprotein',
    'hemoptysis',
    'hemorrhage',
    'hemorrhages',
    'hemorrhagic',
    'hemorrhoid',
    'hemorrhoidectomy',
    'hemorrhoids',
    'hemosiderin',
    'hemosiderosis',
    'hemostasia',
    'hemostasis',
    'hemostat',
    'hemostatic',
    'hemothorax',
    'hemp',
    'hempen',
    'hems',
    'hemstitch',
    'hemstitching',
    'hen',
    'henbane',
    'henbit',
    'henbits',
    'hence',
    'henchman',
    'hencoop',
    'hendiadys',
    'hendrix',
    'henhouse',
    'henna',
    'hennas',
    'henpecked',
    'henri',
    'henries',
    'henrik',
    'henroost',
    'henry',
    'hens',
    'hensel',
    'henson',
    'hep',
    'hepadnavirus',
    'heparin',
    'hepatic',
    'hepatica',
    'hepaticae',
    'hepaticopsida',
    'hepatitis',
    'hepatocarcinoma',
    'hepatoflavin',
    'hepatoma',
    'hepatomegaly',
    'hepatotoxic',
    'hepatotoxin',
    'hepburn',
    'hephaestus',
    'hephaistos',
    'heptad',
    'heptagon',
    'heptane',
    'hepworth',
    'her',
    'hera',
    'heracles',
    'heracleum',
    'heraclitus',
    'herakles',
    'herald',
    'heralded',
    'heraldic',
    'heraldist',
    'heraldry',
    'herat',
    'herb',
    'herbaceous',
    'herbage',
    'herbal',
    'herbalist',
    'herbarium',
    'herbart',
    'herbert',
    'herbicide',
    'herbicides',
    'herbivore',
    'herbivorous',
    'herbs',
    'herculaneum',
    'herculean',
    'hercules',
    'herculius',
    'herd',
    'herded',
    'herder',
    'herders',
    'herding',
    'herds',
    'herdsman',
    'herdsmen',
    'here',
    'hereafter',
    'hereditament',
    'hereditarianism',
    'hereditary',
    'heredity',
    'hereford',
    'hereness',
    'herero',
    'heresies',
    'heresy',
    'heretic',
    'heretical',
    'heretics',
    'heritable',
    'heritage',
    'heritiera',
    'heritor',
    'herm',
    'herman',
    'hermann',
    'hermannia',
    'hermaphrodism',
    'hermaphrodite',
    'hermaphroditic',
    'hermaphroditism',
    'hermaphroditus',
    'hermeneutic',
    'hermeneutics',
    'hermes',
    'hermetic',
    'hermetically',
    'hermissenda',
    'hermit',
    'hermitage',
    'hermitic',
    'hermitical',
    'hermits',
    'hermosillo',
    'hernando',
    'hernaria',
    'hernia',
    'herniation',
    'hero',
    'herod',
    'herodotus',
    'heroes',
    'heroic',
    'heroical',
    'heroics',
    'heroin',
    'heroine',
    'heroism',
    'heron',
    'heronry',
    'herons',
    'heros',
    'herpangia',
    'herpes',
    'herpestes',
    'herpetologist',
    'herpetology',
    'herr',
    'herrenvolk',
    'herrerasaur',
    'herrerasaurus',
    'herrick',
    'herring',
    'herringbone',
    'herrings',
    'herschel',
    'herself',
    'hershey',
    'hertfordshire',
    'hertha',
    'hertz',
    'hertzian',
    'hervey',
    'herzberg',
    'heshvan',
    'hesiod',
    'hesitance',
    'hesitancy',
    'hesitant',
    'hesitantly',
    'hesitate',
    'hesitater',
    'hesitates',
    'hesitating',
    'hesitation',
    'hesitations',
    'hesitator',
    'hesperian',
    'hesperides',
    'hesperiphona',
    'hesperis',
    'hesperus',
    'hess',
    'hesse',
    'hessian',
    'hessonite',
    'hestia',
    'het',
    'heteranthera',
    'heterobasidiomycetes',
    'heterocephalus',
    'heterocercal',
    'heterocycle',
    'heterocyclic',
    'heterodactyl',
    'heterodon',
    'heterodox',
    'heterodoxy',
    'heterodyne',
    'heterodynes',
    'heterodyning',
    'heteroecious',
    'heterogeneity',
    'heterogeneous',
    'heterogeneousness',
    'heterogenesis',
    'heterogenous',
    'heterograft',
    'heteroicous',
    'heterokontae',
    'heterokontophyta',
    'heterologic',
    'heterological',
    'heterologous',
    'heterology',
    'heteromeles',
    'heterometabolic',
    'heterometabolism',
    'heterometabolous',
    'heterometaboly',
    'heteromyidae',
    'heteronym',
    'heteronyms',
    'heterophil',
    'heterophylla',
    'heteroploid',
    'heteroploidy',
    'heteroptera',
    'heteroscelus',
    'heterosexism',
    'heterosexual',
    'heterosexualism',
    'heterosexuality',
    'heterosis',
    'heterosomata',
    'heterosporous',
    'heterospory',
    'heterostracan',
    'heterostraci',
    'heterotaxy',
    'heterotheca',
    'heterothermic',
    'heterotrichales',
    'heterotroph',
    'heterotrophic',
    'heterozygosity',
    'heterozygote',
    'heterozygous',
    'heth',
    'heuchera',
    'heulandite',
    'heuristic',
    'hevea',
    'hevesy',
    'hew',
    'hewer',
    'hewn',
    'hews',
    'hex',
    'hexachlorophene',
    'hexad',
    'hexadecimal',
    'hexadrol',
    'hexagon',
    'hexagonal',
    'hexagons',
    'hexagram',
    'hexagrammidae',
    'hexagrammos',
    'hexahedron',
    'hexalectris',
    'hexameter',
    'hexamita',
    'hexanchidae',
    'hexanchus',
    'hexane',
    'hexangular',
    'hexapod',
    'hexapoda',
    'hexed',
    'hexenbesen',
    'hexestrol',
    'hexose',
    'heyday',
    'heyerdahl',
    'heyrovsky',
    'heyse',
    'heyward',
    'hezbollah',
    'hezekiah',
    'hf',
    'hfc',
    'hg',
    'hhs',
    'hi',
    'hiatus',
    'hiawatha',
    'hibachi',
    'hibbertia',
    'hibbing',
    'hibernal',
    'hibernate',
    'hibernates',
    'hibernating',
    'hibernation',
    'hibernia',
    'hibiscus',
    'hiccough',
    'hiccup',
    'hick',
    'hickey',
    'hickock',
    'hickories',
    'hickory',
    'hicks',
    'hid',
    'hidatsa',
    'hidden',
    'hiddenite',
    'hiddenness',
    'hide',
    'hideaway',
    'hidebound',
    'hideous',
    'hideousness',
    'hideout',
    'hides',
    'hiding',
    'hidrosis',
    'hidrotic',
    'hie',
    'hiemal',
    'hieracium',
    'hierarch',
    'hierarchal',
    'hierarchic',
    'hierarchical',
    'hierarchically',
    'hierarchy',
    'hieratic',
    'hieratical',
    'hierocracy',
    'hieroglyph',
    'hieroglyphic',
    'hieroglyphical',
    'hieroglyphics',
    'hierolatry',
    'hieronymus',
    'hifalutin',
    'higginson',
    'higgle',
    'high',
    'highball',
    'highballs',
    'highbinder',
    'highboard',
    'highborn',
    'highboy',
    'highbrow',
    'highbrowed',
    'highchair',
    'higher',
    'highest',
    'highfalutin',
    'highfaluting',
    'highflier',
    'highflyer',
    'highflying',
    'highjack',
    'highjacker',
    'highjacking',
    'highland',
    'highlander',
    'highlanders',
    'highlands',
    'highlife',
    'highlight',
    'highlighted',
    'highlighter',
    'highlighting',
    'highlights',
    'highly',
    'highness',
    'highroad',
    'highs',
    'highschool',
    'hightail',
    'highwater',
    'highway',
    'highwayman',
    'highways',
    'higi',
    'hijab',
    'hijack',
    'hijacked',
    'hijacker',
    'hijacking',
    'hijaz',
    'hijinks',
    'hike',
    'hiked',
    'hiker',
    'hikers',
    'hiking',
    'hilar',
    'hilarious',
    'hilarity',
    'hilbert',
    'hildebrand',
    'hill',
    'hillary',
    'hillbilly',
    'hillel',
    'hilliness',
    'hillock',
    'hills',
    'hillside',
    'hillsides',
    'hilltop',
    'hilly',
    'hilo',
    'hilt',
    'hilum',
    'hilus',
    'him',
    'himalaya',
    'himalayan',
    'himalayas',
    'himalayish',
    'himantoglossum',
    'himantopus',
    'himmler',
    'himself',
    'himslef',
    'hin',
    'hinault',
    'hinayana',
    'hinayanism',
    'hinayanist',
    'hind',
    'hindbrain',
    'hindemith',
    'hindenburg',
    'hinder',
    'hinderance',
    'hindered',
    'hindering',
    'hindermost',
    'hinders',
    'hindfoot',
    'hindgut',
    'hindi',
    'hindlimb',
    'hindmost',
    'hindoo',
    'hindooism',
    'hindoostani',
    'hindostani',
    'hindquarter',
    'hindquarters',
    'hindrance',
    'hindshank',
    'hindsight',
    'hindu',
    'hinduism',
    'hindus',
    'hindustan',
    'hindustani',
    'hine',
    'hing',
    'hinge',
    'hinged',
    'hinges',
    'hinny',
    'hinson',
    'hint',
    'hinterland',
    'hip',
    'hipbone',
    'hipflask',
    'hiplength',
    'hipless',
    'hipline',
    'hipparchus',
    'hippeastrum',
    'hipped',
    'hippie',
    'hippies',
    'hippo',
    'hippobosca',
    'hippoboscid',
    'hippoboscidae',
    'hippocampus',
    'hippocastanaceae',
    'hippocrates',
    'hippocratic',
    'hippocrepis',
    'hippodamia',
    'hippodrome',
    'hippoglossoides',
    'hippoglossus',
    'hippopotami',
    'hippopotamidae',
    'hippopotamus',
    'hipposideridae',
    'hipposideros',
    'hippotragus',
    'hippy',
    'hips',
    'hipster',
    'hipsters',
    'hipsurus',
    'hircine',
    'hire',
    'hired',
    'hireling',
    'hirer',
    'hires',
    'hiring',
    'hirohito',
    'hiroshima',
    'hirschfeld',
    'hirschsprung',
    'hirsute',
    'hirsuteness',
    'hirsutism',
    'hirudinea',
    'hirudinean',
    'hirudinidae',
    'hirudo',
    'hirundinidae',
    'hirundo',
    'his',
    'hispanic',
    'hispaniola',
    'hispaniolan',
    'hispid',
    'hiss',
    'hisser',
    'hissing',
    'histaminase',
    'histamine',
    'histidine',
    'histiocyte',
    'histiocytosis',
    'histocompatibility',
    'histogram',
    'histoincompatibility',
    'histologic',
    'histological',
    'histologically',
    'histologist',
    'histology',
    'histone',
    'historian',
    'historians',
    'historic',
    'historical',
    'historically',
    'historicalness',
    'historicism',
    'historied',
    'histories',
    'historiographer',
    'historiography',
    'history',
    'histrion',
    'histrionic',
    'histrionics',
    'hit',
    'hitch',
    'hitchcock',
    'hitched',
    'hitches',
    'hitchhike',
    'hitchhiker',
    'hitchings',
    'hitchiti',
    'hitchrack',
    'hitherto',
    'hitler',
    'hitlerian',
    'hitless',
    'hitman',
    'hits',
    'hitter',
    'hitting',
    'hittite',
    'hittites',
    'hiv',
    'hive',
    'hived',
    'hives',
    'hizballah',
    'hizbollah',
    'hizbullah',
    'hl',
    'hm',
    'hmo',
    'hmong',
    'hn',
    'hnd',
    'ho',
    'hoactzin',
    'hoagie',
    'hoagland',
    'hoagy',
    'hoar',
    'hoard',
    'hoarded',
    'hoarder',
    'hoarding',
    'hoards',
    'hoarfrost',
    'hoariness',
    'hoarse',
    'hoarseness',
    'hoary',
    'hoatzin',
    'hoatzins',
    'hoax',
    'hoaxer',
    'hob',
    'hobart',
    'hobbes',
    'hobbies',
    'hobbit',
    'hobble',
    'hobbled',
    'hobbledehoy',
    'hobbler',
    'hobbles',
    'hobbling',
    'hobbs',
    'hobby',
    'hobbyhorse',
    'hobbyism',
    'hobbyist',
    'hobgoblin',
    'hobnail',
    'hobnailed',
    'hobnails',
    'hobnob',
    'hobnobs',
    'hobo',
    'hoboes',
    'hoc',
    'hock',
    'hockey',
    'hod',
    'hodeida',
    'hoder',
    'hodesh',
    'hodgepodge',
    'hodgkin',
    'hodman',
    'hodometer',
    'hodoscope',
    'hodr',
    'hodur',
    'hoe',
    'hoecake',
    'hoehandle',
    'hoeing',
    'hoenir',
    'hoffa',
    'hoffman',
    'hoffmann',
    'hoffmannsthal',
    'hog',
    'hogan',
    'hogarth',
    'hogback',
    'hogchoker',
    'hogfish',
    'hogg',
    'hogged',
    'hogget',
    'hoggish',
    'hoggishness',
    'hogmanay',
    'hognose',
    'hogs',
    'hogshead',
    'hogsheads',
    'hogwash',
    'hogweed',
    'hohenlinden',
    'hohenzollern',
    'hoheria',
    'hohhot',
    'hoijer',
    'hoist',
    'hoisted',
    'hoister',
    'hoisting',
    'hoka',
    'hokan',
    'hokey',
    'hokkaido',
    'hokkianese',
    'hokum',
    'hokusai',
    'holarrhena',
    'holbein',
    'holbrookia',
    'holcus',
    'hold',
    'holdall',
    'holder',
    'holders',
    'holdfast',
    'holding',
    'holdings',
    'holdout',
    'holdover',
    'holds',
    'holdup',
    'hole',
    'holed',
    'holes',
    'holey',
    'holibut',
    'holiday',
    'holidaymaker',
    'holidays',
    'holiest',
    'holiness',
    'holing',
    'holism',
    'holistic',
    'holla',
    'holland',
    'hollandaise',
    'hollander',
    'hollands',
    'holler',
    'hollered',
    'hollering',
    'hollerith',
    'hollo',
    'holloa',
    'hollow',
    'holloware',
    'hollowed',
    'hollowing',
    'hollowness',
    'hollows',
    'hollowware',
    'holly',
    'hollygrape',
    'hollyhock',
    'hollywood',
    'holm',
    'holmes',
    'holmium',
    'holocaust',
    'holocene',
    'holocentridae',
    'holocentrus',
    'holocephalan',
    'holocephali',
    'holocephalian',
    'holofernes',
    'hologram',
    'holograms',
    'holograph',
    'holographic',
    'holographical',
    'holography',
    'holometabola',
    'holometabolic',
    'holometabolism',
    'holometabolous',
    'holometaboly',
    'holonym',
    'holonymy',
    'holophyte',
    'holophytic',
    'holothuria',
    'holothurian',
    'holothuridae',
    'holothuroidea',
    'holotype',
    'holozoic',
    'holstein',
    'holster',
    'holy',
    'holystone',
    'homage',
    'homaridae',
    'homarus',
    'hombre',
    'homburg',
    'home',
    'homebody',
    'homebound',
    'homeboy',
    'homebrew',
    'homebuilder',
    'homecoming',
    'homefolk',
    'homegirl',
    'homegrown',
    'homel',
    'homeland',
    'homeless',
    'homelessness',
    'homelike',
    'homeliness',
    'homely',
    'homemade',
    'homemaker',
    'homemaking',
    'homeobox',
    'homeopath',
    'homeopathic',
    'homeopathy',
    'homeostasis',
    'homeostatic',
    'homeotherm',
    'homeothermic',
    'homeotic',
    'homeowner',
    'homepage',
    'homer',
    'homeric',
    'homeroom',
    'homes',
    'homesick',
    'homesickness',
    'homespun',
    'homestead',
    'homesteader',
    'homesteads',
    'homestretch',
    'hometown',
    'homeward',
    'homework',
    'homey',
    'homicidal',
    'homicide',
    'homiletic',
    'homiletical',
    'homiletics',
    'homilies',
    'homily',
    'hominal',
    'hominem',
    'homing',
    'hominian',
    'hominid',
    'hominidae',
    'hominids',
    'hominine',
    'hominoid',
    'hominoidea',
    'hominoids',
    'hominy',
    'hommos',
    'homo',
    'homobasidiomycetes',
    'homocentric',
    'homocercal',
    'homochromatic',
    'homocyclic',
    'homoecious',
    'homoeopath',
    'homoeopathy',
    'homoerotic',
    'homoeroticism',
    'homogenate',
    'homogeneity',
    'homogeneous',
    'homogeneousness',
    'homogenisation',
    'homogenise',
    'homogenised',
    'homogenization',
    'homogenize',
    'homogenized',
    'homogenous',
    'homogeny',
    'homograft',
    'homograph',
    'homographs',
    'homogyne',
    'homoiotherm',
    'homoiothermic',
    'homologic',
    'homological',
    'homologise',
    'homologize',
    'homologous',
    'homology',
    'homomorphism',
    'homomorphy',
    'homona',
    'homonym',
    'homonymic',
    'homonymous',
    'homonyms',
    'homonymy',
    'homophile',
    'homophobe',
    'homophobia',
    'homophobic',
    'homophone',
    'homophones',
    'homophonic',
    'homophonous',
    'homophony',
    'homoptera',
    'homopteran',
    'homopterous',
    'homosexual',
    'homosexualism',
    'homosexuality',
    'homosexuals',
    'homosporous',
    'homospory',
    'homostyled',
    'homostylic',
    'homostylous',
    'homotherm',
    'homothermic',
    'homozygosity',
    'homozygote',
    'homozygous',
    'homunculus',
    'homy',
    'homyel',
    'honcho',
    'honda',
    'hondo',
    'honduran',
    'honduras',
    'hone',
    'honegger',
    'honest',
    'honestly',
    'honestness',
    'honesty',
    'honey',
    'honeybee',
    'honeybees',
    'honeybells',
    'honeycomb',
    'honeycombed',
    'honeycombs',
    'honeycreeper',
    'honeycreepers',
    'honeydew',
    'honeyed',
    'honeyflower',
    'honeylike',
    'honeymoon',
    'honeymooner',
    'honeypot',
    'honeysucker',
    'honeysuckle',
    'honeysuckles',
    'hong',
    'honiara',
    'honied',
    'honk',
    'honker',
    'honkey',
    'honkie',
    'honking',
    'honky',
    'honkytonk',
    'honolulu',
    'honor',
    'honorable',
    'honorableness',
    'honorably',
    'honorarium',
    'honorary',
    'honore',
    'honored',
    'honoree',
    'honorific',
    'honoring',
    'honors',
    'honour',
    'honourable',
    'honourableness',
    'honours',
    'honshu',
    'hooch',
    'hood',
    'hooded',
    'hoodlum',
    'hoodmold',
    'hoodmould',
    'hoodoo',
    'hoodooism',
    'hoods',
    'hoodwink',
    'hoodwinked',
    'hooey',
    'hoof',
    'hoofed',
    'hoofer',
    'hoofing',
    'hooflike',
    'hoofprint',
    'hoofs',
    'hook',
    'hookah',
    'hooke',
    'hooked',
    'hooker',
    'hooking',
    'hooklike',
    'hooknose',
    'hooks',
    'hookup',
    'hookups',
    'hookworm',
    'hookworms',
    'hooky',
    'hooligan',
    'hooliganism',
    'hoop',
    'hooped',
    'hoopla',
    'hoopoe',
    'hoopoes',
    'hoopoo',
    'hoops',
    'hoopskirt',
    'hooray',
    'hoosegow',
    'hoosgow',
    'hoosier',
    'hoot',
    'hootch',
    'hooter',
    'hooting',
    'hoots',
    'hooved',
    'hoover',
    'hooves',
    'hop',
    'hope',
    'hoped',
    'hopeful',
    'hopefully',
    'hopefulness',
    'hopefuls',
    'hopeh',
    'hopei',
    'hopeless',
    'hopelessly',
    'hopelessness',
    'hoper',
    'hopes',
    'hopi',
    'hoping',
    'hopkins',
    'hopkinson',
    'hopped',
    'hopper',
    'hoppers',
    'hopping',
    'hopple',
    'hops',
    'hopsack',
    'hopsacking',
    'hopscotch',
    'horace',
    'horary',
    'horatio',
    'horde',
    'hordeolum',
    'hordes',
    'hordeum',
    'horehound',
    'horizon',
    'horizons',
    'horizontal',
    'horizontality',
    'horizontally',
    'horizonte',
    'hormonal',
    'hormone',
    'hormones',
    'horn',
    'hornbeam',
    'hornbeams',
    'hornbill',
    'hornbills',
    'hornblende',
    'hornblower',
    'hornbook',
    'horne',
    'horned',
    'horneophyton',
    'hornet',
    'hornets',
    'horney',
    'hornfels',
    'horniness',
    'hornist',
    'hornless',
    'hornlike',
    'hornpipe',
    'hornpout',
    'horns',
    'hornstone',
    'hornswoggle',
    'hornworm',
    'hornworms',
    'hornwort',
    'hornworts',
    'horny',
    'horologe',
    'horologer',
    'horologist',
    'horologists',
    'horology',
    'horoscope',
    'horoscopes',
    'horoscopy',
    'horowitz',
    'horrendous',
    'horrible',
    'horrid',
    'horridness',
    'horrific',
    'horrified',
    'horrify',
    'horrifying',
    'horripilate',
    'horripilation',
    'horror',
    'horrors',
    'hors',
    'horse',
    'horseback',
    'horsebean',
    'horsebox',
    'horsecar',
    'horsecloth',
    'horsefish',
    'horseflesh',
    'horseflies',
    'horsefly',
    'horsehair',
    'horsehead',
    'horsehide',
    'horselaugh',
    'horseleech',
    'horseless',
    'horseman',
    'horsemanship',
    'horsemeat',
    'horsemen',
    'horsemint',
    'horseplay',
    'horsepond',
    'horsepower',
    'horseradish',
    'horses',
    'horseshit',
    'horseshoe',
    'horseshoer',
    'horseshoes',
    'horseshow',
    'horsetail',
    'horsetails',
    'horseweed',
    'horsewhip',
    'horsewhipping',
    'horsewoman',
    'horsing',
    'horst',
    'horta',
    'hortative',
    'hortatory',
    'hortensia',
    'horticultural',
    'horticulturally',
    'horticulture',
    'horticulturist',
    'horus',
    'hosanna',
    'hose',
    'hosea',
    'hosepipe',
    'hosier',
    'hosiery',
    'hospice',
    'hospitable',
    'hospitableness',
    'hospital',
    'hospitalisation',
    'hospitalise',
    'hospitality',
    'hospitalization',
    'hospitalize',
    'hospitalized',
    'hospitals',
    'host',
    'hosta',
    'hostaceae',
    'hostage',
    'hostages',
    'hosted',
    'hostel',
    'hosteller',
    'hostelry',
    'hostess',
    'hostile',
    'hostilities',
    'hostility',
    'hosting',
    'hostler',
    'hosts',
    'hot',
    'hotbed',
    'hotbox',
    'hotcake',
    'hotchpotch',
    'hotdog',
    'hotei',
    'hotel',
    'hotelier',
    'hotelkeeper',
    'hotelman',
    'hotels',
    'hotfoot',
    'hoth',
    'hothead',
    'hotheaded',
    'hothouse',
    'hothr',
    'hotly',
    'hotness',
    'hotplate',
    'hotpot',
    'hotshot',
    'hotspot',
    'hotspur',
    'hottentot',
    'hottish',
    'hottonia',
    'houdah',
    'houdini',
    'houghton',
    'houhere',
    'hoummos',
    'hound',
    'hounded',
    'hounds',
    'hour',
    'hourglass',
    'houri',
    'hourlong',
    'hourly',
    'hours',
    'housatonic',
    'house',
    'houseboat',
    'housebound',
    'housebreak',
    'housebreaker',
    'housebreaking',
    'housebroken',
    'housebuilder',
    'houseclean',
    'housecleaning',
    'housecleans',
    'housecoat',
    'housecraft',
    'housed',
    'housedog',
    'housefather',
    'houseflies',
    'housefly',
    'houseful',
    'houseguest',
    'houseguests',
    'household',
    'householder',
    'households',
    'househusband',
    'housekeep',
    'housekeeper',
    'housekeeping',
    'houselights',
    'housemaid',
    'houseman',
    'housemaster',
    'housemate',
    'housemother',
    'housepaint',
    'houseplant',
    'houseplants',
    'houseroom',
    'houses',
    'housetop',
    'housewarming',
    'housewife',
    'housewifely',
    'housewifery',
    'housework',
    'housewrecker',
    'housing',
    'housman',
    'houston',
    'houttuynia',
    'houyhnhnm',
    'houyhnhnms',
    'hove',
    'hovea',
    'hovel',
    'hover',
    'hovercraft',
    'hovering',
    'hovers',
    'how',
    'howard',
    'howdah',
    'howdy',
    'howe',
    'howells',
    'however',
    'howitzer',
    'howl',
    'howler',
    'howling',
    'hoy',
    'hoya',
    'hoyden',
    'hoydenish',
    'hoydenism',
    'hoyle',
    'hp',
    'hq',
    'hr',
    'hrolf',
    'hrt',
    'hrvatska',
    'hryvnia',
    'hs',
    'hsian',
    'html',
    'http',
    'hua',
    'huainaputina',
    'hualapai',
    'hualpai',
    'huamachil',
    'huambo',
    'huarache',
    'huaraches',
    'huascaran',
    'hub',
    'hubbard',
    'hubble',
    'hubbub',
    'hubby',
    'hubcap',
    'hubel',
    'hubris',
    'huck',
    'huckaback',
    'huckleberries',
    'huckleberry',
    'huckster',
    'hud',
    'huddle',
    'huddled',
    'huddler',
    'huddlers',
    'hudood',
    'hudson',
    'hudsonia',
    'hudud',
    'hue',
    'hued',
    'hueless',
    'huff',
    'huffed',
    'huffiness',
    'huffing',
    'huffish',
    'huffishness',
    'huffy',
    'hufstedler',
    'hug',
    'huge',
    'hugged',
    'hugger',
    'hugging',
    'huggins',
    'hugh',
    'hughes',
    'hugo',
    'hugoesque',
    'hugs',
    'hugueninia',
    'huguenot',
    'huguenots',
    'huisache',
    'huitre',
    'huji',
    'hula',
    'hulk',
    'hulking',
    'hulky',
    'hull',
    'hullabaloo',
    'hulled',
    'hullo',
    'hulls',
    'hulsea',
    'hum',
    'human',
    'humane',
    'humaneness',
    'humanisation',
    'humanise',
    'humanism',
    'humanist',
    'humanistic',
    'humanists',
    'humanitarian',
    'humanitarianism',
    'humanities',
    'humanity',
    'humanization',
    'humanize',
    'humankind',
    'humanlike',
    'humanness',
    'humanoid',
    'humans',
    'humate',
    'humber',
    'humble',
    'humblebee',
    'humbled',
    'humbleness',
    'humbler',
    'humbles',
    'humbling',
    'humbly',
    'humboldt',
    'humbug',
    'humdinger',
    'humdrum',
    'hume',
    'humectant',
    'humeri',
    'humerus',
    'humic',
    'humid',
    'humidifier',
    'humidifies',
    'humidify',
    'humidity',
    'humidness',
    'humification',
    'humified',
    'humify',
    'humiliate',
    'humiliated',
    'humiliating',
    'humiliation',
    'humility',
    'humin',
    'hummed',
    'hummer',
    'humming',
    'hummingbird',
    'hummingbirds',
    'hummock',
    'hummus',
    'humongous',
    'humor',
    'humoral',
    'humoring',
    'humorist',
    'humorless',
    'humorous',
    'humorously',
    'humorousness',
    'humors',
    'humour',
    'humourist',
    'humourless',
    'humourous',
    'humous',
    'hump',
    'humpback',
    'humpbacked',
    'humpbacks',
    'humped',
    'humperdinck',
    'humphrey',
    'humping',
    'humps',
    'humpty',
    'humulin',
    'humulus',
    'humus',
    'humvee',
    'hun',
    'hunan',
    'hunch',
    'hunchback',
    'hunchbacked',
    'hunched',
    'hundred',
    'hundreds',
    'hundredth',
    'hundredths',
    'hundredweight',
    'hung',
    'hungarian',
    'hungarians',
    'hungary',
    'hunger',
    'hungriness',
    'hungry',
    'hunk',
    'hunker',
    'hunkered',
    'hunkpapa',
    'hunnemannia',
    'hunt',
    'hunted',
    'hunter',
    'hunters',
    'hunting',
    'huntington',
    'huntress',
    'hunts',
    'huntsman',
    'huntsmen',
    'huntsville',
    'hupa',
    'hurdle',
    'hurdler',
    'hurdles',
    'hurdling',
    'hurl',
    'hurled',
    'hurler',
    'hurling',
    'hurok',
    'huron',
    'hurrah',
    'hurricane',
    'hurricanes',
    'hurried',
    'hurriedly',
    'hurriedness',
    'hurry',
    'hurrying',
    'hurt',
    'hurtful',
    'hurting',
    'hurtle',
    'hurtled',
    'hus',
    'husain',
    'husayn',
    'husband',
    'husbandly',
    'husbandman',
    'husbandry',
    'husbands',
    'hush',
    'hushed',
    'hushing',
    'hushpuppy',
    'husk',
    'husked',
    'huskiness',
    'husking',
    'husks',
    'husky',
    'huss',
    'hussar',
    'hussein',
    'husserl',
    'hussite',
    'hussy',
    'hustings',
    'hustle',
    'hustled',
    'hustler',
    'huston',
    'hut',
    'hutch',
    'hutchins',
    'hutchinson',
    'hutment',
    'huts',
    'hutton',
    'hutu',
    'hutus',
    'hutzpah',
    'huxleian',
    'huxley',
    'huxleyan',
    'huygens',
    'hyacinth',
    'hyacinthaceae',
    'hyacinthoides',
    'hyacinths',
    'hyades',
    'hyaena',
    'hyaenidae',
    'hyalin',
    'hyaline',
    'hyalinisation',
    'hyalinization',
    'hyaloid',
    'hyalophora',
    'hyaloplasm',
    'hyaloplasmic',
    'hyalosperma',
    'hyalospongiae',
    'hyaluronic',
    'hyaluronidase',
    'hyazyme',
    'hybanthus',
    'hybrid',
    'hybridisation',
    'hybridise',
    'hybridization',
    'hybridize',
    'hybridized',
    'hybridizing',
    'hybridoma',
    'hybrids',
    'hydantoin',
    'hydathode',
    'hydatid',
    'hydatidosis',
    'hyderabad',
    'hydnaceae',
    'hydnocarpus',
    'hydnoraceae',
    'hydnum',
    'hydra',
    'hydralazine',
    'hydramnios',
    'hydrangea',
    'hydrangeaceae',
    'hydrant',
    'hydrargyrum',
    'hydrarthrosis',
    'hydras',
    'hydrastis',
    'hydrate',
    'hydrated',
    'hydration',
    'hydraulic',
    'hydraulically',
    'hydraulics',
    'hydrazine',
    'hydrazoic',
    'hydrazoite',
    'hydremia',
    'hydric',
    'hydride',
    'hydrilla',
    'hydriodic',
    'hydrobates',
    'hydrobatidae',
    'hydrobromic',
    'hydrocarbon',
    'hydrocarbons',
    'hydrocele',
    'hydrocephalic',
    'hydrocephalus',
    'hydrocephaly',
    'hydrocharidaceae',
    'hydrocharis',
    'hydrocharitaceae',
    'hydrochloric',
    'hydrochloride',
    'hydrochlorofluorocarbon',
    'hydrochlorothiazide',
    'hydrochoeridae',
    'hydrochoerus',
    'hydrocolloid',
    'hydrocortisone',
    'hydrocortone',
    'hydrocracking',
    'hydrocyanic',
    'hydrodamalis',
    'hydrodiuril',
    'hydrodynamic',
    'hydrodynamics',
    'hydroelectric',
    'hydroelectricity',
    'hydroflumethiazide',
    'hydrofluoric',
    'hydrofluorocarbon',
    'hydrofoil',
    'hydrogel',
    'hydrogen',
    'hydrogenate',
    'hydrogenated',
    'hydrogenating',
    'hydrogenation',
    'hydrographic',
    'hydrographical',
    'hydrography',
    'hydroid',
    'hydroiodic',
    'hydrokinetic',
    'hydrokinetics',
    'hydrolise',
    'hydrolith',
    'hydrolize',
    'hydrolized',
    'hydrolizing',
    'hydrologist',
    'hydrology',
    'hydrolysate',
    'hydrolysates',
    'hydrolyse',
    'hydrolyses',
    'hydrolysis',
    'hydrolyzable',
    'hydrolyze',
    'hydrolyzed',
    'hydrolyzes',
    'hydrolyzing',
    'hydromancer',
    'hydromancy',
    'hydromantes',
    'hydromel',
    'hydrometer',
    'hydrometric',
    'hydrometry',
    'hydromorphone',
    'hydromyinae',
    'hydromys',
    'hydronephrosis',
    'hydropathic',
    'hydropathy',
    'hydrophidae',
    'hydrophilic',
    'hydrophobia',
    'hydrophobic',
    'hydrophobicity',
    'hydrophyllaceae',
    'hydrophyllum',
    'hydrophyte',
    'hydrophytic',
    'hydroplane',
    'hydroponic',
    'hydroponics',
    'hydrops',
    'hydrosphere',
    'hydrostatic',
    'hydrostatics',
    'hydrotherapy',
    'hydrothorax',
    'hydrous',
    'hydroxide',
    'hydroxides',
    'hydroxy',
    'hydroxybenzene',
    'hydroxybutyrate',
    'hydroxybutyric',
    'hydroxychloroquine',
    'hydroxyl',
    'hydroxymethyl',
    'hydroxyproline',
    'hydroxytetracycline',
    'hydroxyzine',
    'hydrozoa',
    'hydrozoan',
    'hydrozoans',
    'hydrus',
    'hyemoschus',
    'hyena',
    'hyenas',
    'hygeia',
    'hygiene',
    'hygienic',
    'hygienical',
    'hygienics',
    'hygienise',
    'hygienist',
    'hygienize',
    'hygrocybe',
    'hygrodeik',
    'hygrometer',
    'hygrophoraceae',
    'hygrophorus',
    'hygrophyte',
    'hygrophytic',
    'hygroscope',
    'hygroscopic',
    'hygroton',
    'hygrotrama',
    'hyla',
    'hylactophryne',
    'hylidae',
    'hylobates',
    'hylobatidae',
    'hylocereus',
    'hylocichla',
    'hylophylax',
    'hymen',
    'hymenaea',
    'hymenal',
    'hymenanthera',
    'hymeneal',
    'hymeneals',
    'hymenium',
    'hymenogastrales',
    'hymenomycetes',
    'hymenophyllaceae',
    'hymenophyllum',
    'hymenopter',
    'hymenoptera',
    'hymenopteran',
    'hymenopterans',
    'hymenopteron',
    'hymenopterous',
    'hymie',
    'hymn',
    'hymnal',
    'hymnary',
    'hymnbook',
    'hymned',
    'hymnody',
    'hymns',
    'hynerpeton',
    'hyoid',
    'hyoscine',
    'hyoscyamine',
    'hyoscyamus',
    'hypaethral',
    'hypallage',
    'hypanthium',
    'hypatia',
    'hype',
    'hypentelium',
    'hyperacidity',
    'hyperactive',
    'hyperactivity',
    'hyperacusia',
    'hyperacusis',
    'hyperadrenalism',
    'hyperadrenocorticism',
    'hyperaemia',
    'hyperaldosteronism',
    'hyperalimentation',
    'hyperbaton',
    'hyperbetalipoproteinemia',
    'hyperbilirubinemia',
    'hyperbola',
    'hyperbole',
    'hyperbolic',
    'hyperbolise',
    'hyperbolize',
    'hyperboloid',
    'hyperboloidal',
    'hyperborean',
    'hypercalcaemia',
    'hypercalcemia',
    'hypercalcinuria',
    'hypercalciuria',
    'hypercapnia',
    'hypercarbia',
    'hypercatalectic',
    'hypercellularity',
    'hypercholesteremia',
    'hypercholesterolemia',
    'hypercoaster',
    'hypercritical',
    'hyperdactyly',
    'hyperemesis',
    'hyperemia',
    'hyperemic',
    'hyperextend',
    'hyperextension',
    'hyperfine',
    'hyperglycaemia',
    'hyperglycemia',
    'hyperhidrosis',
    'hypericaceae',
    'hypericales',
    'hypericism',
    'hypericum',
    'hyperidrosis',
    'hyperion',
    'hyperkalemia',
    'hyperlink',
    'hyperlipaemia',
    'hyperlipemia',
    'hyperlipidaemia',
    'hyperlipidemia',
    'hyperlipoidaemia',
    'hyperlipoidemia',
    'hyperlipoproteinemia',
    'hypermarket',
    'hypermastigina',
    'hypermastigote',
    'hypermedia',
    'hypermenorrhea',
    'hypermetropia',
    'hypermetropic',
    'hypermetropy',
    'hypermotility',
    'hypernatremia',
    'hypernym',
    'hypernymy',
    'hyperoartia',
    'hyperodontidae',
    'hyperoglyphe',
    'hyperon',
    'hyperoodon',
    'hyperope',
    'hyperopia',
    'hyperopic',
    'hyperotreta',
    'hyperparathyroidism',
    'hyperpiesia',
    'hyperpiesis',
    'hyperpigmentation',
    'hyperpituitarism',
    'hyperplasia',
    'hyperpnea',
    'hyperpyrexia',
    'hypersecretion',
    'hypersensitised',
    'hypersensitive',
    'hypersensitivity',
    'hypersensitized',
    'hypersomnia',
    'hypersplenism',
    'hyperstat',
    'hypertensin',
    'hypertension',
    'hypertensive',
    'hypertext',
    'hyperthermal',
    'hyperthermia',
    'hyperthermy',
    'hyperthyroidism',
    'hypertonia',
    'hypertonic',
    'hypertonicity',
    'hypertonus',
    'hypertrophied',
    'hypertrophy',
    'hypervelocity',
    'hyperventilate',
    'hyperventilation',
    'hypervitaminosis',
    'hypervolaemia',
    'hypervolemia',
    'hypesthesia',
    'hypethral',
    'hypha',
    'hyphae',
    'hyphantria',
    'hyphema',
    'hyphen',
    'hyphenate',
    'hyphenation',
    'hyphens',
    'hypnagogic',
    'hypnagogue',
    'hypnoanalysis',
    'hypnogenesis',
    'hypnogogic',
    'hypnoid',
    'hypnopedia',
    'hypnophobia',
    'hypnos',
    'hypnosis',
    'hypnotherapy',
    'hypnotic',
    'hypnotically',
    'hypnotics',
    'hypnotise',
    'hypnotised',
    'hypnotiser',
    'hypnotism',
    'hypnotist',
    'hypnotize',
    'hypnotized',
    'hypnotizer',
    'hypo',
    'hypoactive',
    'hypoadrenalism',
    'hypoadrenocorticism',
    'hypobasidium',
    'hypobetalipoproteinemia',
    'hypoblast',
    'hypocalcaemia',
    'hypocalcemia',
    'hypocapnia',
    'hypocellularity',
    'hypochaeris',
    'hypochlorite',
    'hypochlorous',
    'hypochoeris',
    'hypochondria',
    'hypochondriac',
    'hypochondriacal',
    'hypochondriasis',
    'hypochondrium',
    'hypocorism',
    'hypocreaceae',
    'hypocreales',
    'hypocrisy',
    'hypocrite',
    'hypocritical',
    'hypocritically',
    'hypocycloid',
    'hypoderma',
    'hypodermal',
    'hypodermatidae',
    'hypodermic',
    'hypodermis',
    'hypoesthesia',
    'hypogammaglobulinemia',
    'hypoglossal',
    'hypoglycaemia',
    'hypoglycaemic',
    'hypoglycemia',
    'hypoglycemic',
    'hypognathous',
    'hypogonadism',
    'hypokalemia',
    'hypolipoproteinemia',
    'hyponatremia',
    'hyponym',
    'hyponymy',
    'hypopachus',
    'hypoparathyroidism',
    'hypophyseal',
    'hypophysectomise',
    'hypophysectomised',
    'hypophysectomize',
    'hypophysectomized',
    'hypophysectomy',
    'hypophysial',
    'hypophysis',
    'hypopigmentation',
    'hypopitys',
    'hypoplasia',
    'hypopnea',
    'hypoproteinemia',
    'hyposmia',
    'hypospadias',
    'hypostasis',
    'hypostatisation',
    'hypostatise',
    'hypostatization',
    'hypostatize',
    'hypotension',
    'hypotensive',
    'hypotenuse',
    'hypothalamic',
    'hypothalamus',
    'hypothecate',
    'hypothermia',
    'hypothermic',
    'hypotheses',
    'hypothesis',
    'hypothesise',
    'hypothesize',
    'hypothesized',
    'hypothetic',
    'hypothetical',
    'hypothrombinemia',
    'hypothyroidism',
    'hypotonia',
    'hypotonic',
    'hypotonicity',
    'hypotonus',
    'hypovitaminosis',
    'hypovolaemia',
    'hypovolaemic',
    'hypovolemia',
    'hypovolemic',
    'hypoxia',
    'hypoxidaceae',
    'hypoxis',
    'hypozeugma',
    'hypozeuxis',
    'hypsiglena',
    'hypsiprymnodon',
    'hypsography',
    'hypsometer',
    'hypsometry',
    'hyracoidea',
    'hyracotherium',
    'hyrax',
    'hyraxes',
    'hyson',
    'hyssop',
    'hyssopus',
    'hysterectomy',
    'hysteresis',
    'hysteria',
    'hysteric',
    'hysterical',
    'hysterics',
    'hysterocatalepsy',
    'hysterosalpingogram',
    'hysteroscopy',
    'hysterotomy',
    'hystricidae',
    'hystricomorpha',
    'hytrin',
    'hz']



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Orchestrate Sentence Segmentation """


from typing import List
from functools import lru_cache
from lingpatlab.baseblock import BaseObject, Stopwatch, Enforcer

from lingpatlab.segmenter.svc import (
    PerformParagraphSegmentation,
    PerformSentenceSegmentation
)


class Segmenter(BaseObject):
    """ Orchestrate Sentence Segmentation """

    def __init__(self):
        """ Change Log

        Created:
            30-Sept-2021
            craigtrim@gmail.com
            *   created
        Updated:
            27-Mar-2024
            craigtrim@gmail.com
            *   migrated out of modai in pursuit of
                https://github.com/craigtrim/datapipe-apis/issues/72
        """
        BaseObject.__init__(self, __name__)
        self._segment_paragraphs = PerformParagraphSegmentation().process
        self._segment_sentences = PerformSentenceSegmentation().process

    def _input_text(self,
                    input_text: str) -> list:
        paragraphs = []

        for paragraph in self._segment_paragraphs(input_text):
            paragraphs.append(self._segment_sentences(paragraph))

        return paragraphs

    @lru_cache(maxsize=1024, typed=True)
    def input_text(self,
                   input_text: str) -> List[str]:
        """Segment Input Text into Paragraphs and Sentences

        Args:
            input_text (str): An input string of any length or type

        Raises:
            ValueError: input must be a string

        Returns:
            list:   returns a list of lists.
                    Each outer list is a paragraph.
                    Each inner list contains 1..* sentences
        """

        if self.isEnabledForDebug:
            Enforcer.is_str(input_text)

        sw = Stopwatch()

        paragraphs = self._input_text(input_text)

        if self.isEnabledForInfo:
            Enforcer.is_list_of_str(paragraphs)
            self.logger.info('\n'.join([
                "Segmentation of Input Text Complete",
                f"\tTotal Paragraphs: {len(paragraphs)}",
                f"\tTotal Time: {str(sw)}"]))

        return paragraphs



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Extract People Patterns from Text """


from typing import List, Dict
from lingpatlab.utils.dto import Sentence
from lingpatlab.baseblock import BaseObject, Stopwatch


PATTERNS_1: List[str] = [
    'PERSON PERSON',
    'PERSON PERSON PERSON',
    'PERSON PERSON PERSON PERSON',
    'PERSON PERSON PERSON PERSON PERSON',

    'PROPN PERSON PERSON',
    'PROPN PROPN PERSON PERSON',

    'PROPN PROPN PERSON PERSON',
    'PROPN PROPN PERSON PERSON PERSON',
    'PROPN PROPN PERSON PERSON PERSON PERSON',
    'PROPN PROPN PERSON PERSON PERSON PERSON PERSON',

    # Rear/PROPN Admiral/PROPN William/PERSON F/PERSON ./PUNCT Halsey/PERSON Jr/PERSON
    # reference case: tests/extract_people_08_test.py
    'PROPN PROPN PERSON PERSON PUNCT PERSON PERSON',

    # Rear/PROPN Admiral/PROPN William/PERSON S/PERSON ./PUNCT Pye/NOUN
    # reference case: tests/extract_people_10_test.py
    'PROPN PROPN PERSON PERSON PUNCT NOUN',

    # Kelly/PROPN/PERSON Turner/PROPN/ORG
    # reference case: regression/inputs/People-0003.txt
    'PERSON ORG',

    # Rear/PROPN Admiral/PROPN William/PERSON S/PERSON ./PUNCT Pye/AUX
    # reference case: regression/inputs/People-0005.txt
    'PROPN PROPN PERSON PERSON PUNCT AUX'
]


PATTERNS_2: List[str] = [
    '* PERSON',
    '* PERSON PERSON',
    'PERSON * PERSON',
    '* PERSON PERSON PERSON',
    'PERSON PERSON * PERSON PERSON',
    '* PERSON PERSON PERSON PERSON',
    'PERSON PERSON PERSON * PERSON',
]


class PeopleSequenceExtractor(BaseObject):
    """ Extract Topic Patterns from Text """

    def __init__(self,
                 isEnabledForTrace: bool = False):
        """ Change Log

        Created:
            22-Mar-2024
            craigtrim@gmail.com
            *   based on 'person-sequence-extractor'
                https://github.com/craigtrim/datapipe-apis/issues/63
            *   fix defect in PERSON entity matching
                https://github.com/craigtrim/datapipe-apis/issues/65
        """
        BaseObject.__init__(self, __name__)
        self.isEnabledForTrace = isEnabledForTrace

    @staticmethod
    def _cleanse(input_text: str) -> str:
        """ Cleanse the input text """
        while '  ' in input_text:
            input_text = input_text.replace('  ', ' ')

        input_text = input_text.replace("'s", "")

        return input_text.strip()

    def _create_matching_sequence(self,
                                  sentence: Sentence,
                                  pattern: List[str],
                                  i: int) -> str:
        """
        Create a matching sequence from the given sentence, pattern, and index.

        Args:
            sentence (Sentence): The sentence object.
            pattern (List[str]): The pattern to match.
            i (int): The starting index of the matching sequence.

        Returns:
            str: The matching sequence.

        """
        matching_sequence = ' '.join([
            token.text
            for token in sentence.tokens[i:i+len(pattern)]
        ]).strip()

        while '  ' in matching_sequence:
            matching_sequence = matching_sequence.replace(
                '  ', ' ').strip()

        while ' . ' in matching_sequence:
            matching_sequence = matching_sequence.replace(
                ' . ', '. ').strip()

        if self.isEnabledForDebug:
            self.logger.debug(
                f"Found Pattern Match: `{matching_sequence}` from {pattern}")

        return matching_sequence

    def _extract_patterns(self,
                          sentence: Sentence,
                          patterns: List[str]) -> List[str]:
        """
        Extract sequences from the given data based on the provided pattern.

        Args:
            data (List[List[str]]): The input data in List[List[str]] format.
            pattern (str): The pattern to match.

        Returns:
            List[List[str]]: List of sequences that match the given pattern.
        """

        matching_sequences = []

        for pattern in patterns:

            pattern: List[str] = pattern.split()

            for i, _ in enumerate(sentence.tokens):

                if len(pattern) + i > sentence.size():
                    continue

                if sentence.tokens[i].pos in ['DET']:
                    continue

                if self.isEnabledForTrace:
                    self.logger.debug(
                        f"Begin analysis at i={i} with {pattern}")

                def has_match() -> bool:

                    for j in range(len(pattern)):

                        if self.isEnabledForTrace:
                            self.logger.debug(
                                f"Analyze Token: (text={sentence.tokens[i + j].text}), (pos={sentence.tokens[i + j].pos}), (ent={sentence.tokens[i + j].ent}) for {pattern[j]}")

                        def has_pos_match() -> bool:
                            return pattern[j] != sentence.tokens[i + j].pos

                        def has_ent_match() -> bool:
                            return pattern[j] != sentence.tokens[i + j].ent

                        # don't match wildcards at the start of a sentence; anecdotal observation
                        if pattern[j] == '*' and i != 0:
                            continue

                        # ------------------------------------------------------------------
                        # Purpose:      The 'smoothing' of PROPN/NOUN holds true when
                        #               extracting topics, but not people
                        # Updated:      27-Mar-2024
                        #
                        # elif pattern[j] == 'NOUN' and sentence.tokens[i + j].pos == 'PROPN':
                        #     continue
                        # elif pattern[j] == 'PROPN' and sentence.tokens[i + j].pos == 'NOUN':
                        #     continue
                        # ------------------------------------------------------------------

                        elif pattern[j] == 'PUNCT' and sentence.tokens[i + j].text.strip() == '.':
                            continue

                        elif not has_ent_match() and not has_pos_match():
                            if self.isEnabledForTrace:
                                self.logger.debug(
                                    f"\tToken Failure: (text={sentence.tokens[i + j].text}), (pos={sentence.tokens[i + j].pos}), (ent={sentence.tokens[i + j].ent}) for {pattern[j]}")
                            return False

                        if pattern[j] != sentence.tokens[i + j].pos and pattern[j] != sentence.tokens[i + j].ent:
                            return False

                    def has_person_ent() -> bool:
                        # ------------------------------------------------------------------
                        # Purpose:      Ensure that the sequence has enough PERSON entities
                        # Reference:    https://github.com/craigtrim/datapipe-apis/issues/65
                        # Updated:      22-Mar-2024
                        # ------------------------------------------------------------------
                        person_ctr = 0
                        for j in range(len(pattern)):
                            if sentence.tokens[i + j].ent == 'PERSON':
                                person_ctr += 1
                        if len(pattern) <= 3 and person_ctr >= 1:
                            return True
                        if len(pattern) <= 6 and person_ctr >= 2:
                            return True
                        return person_ctr >= 3

                    return has_person_ent()

                if has_match():
                    matching_sequences.append(self._create_matching_sequence(
                        sentence=sentence, pattern=pattern, i=i))

        def is_upper(ch: str) -> bool:
            if ch in [
                '.'
            ]:
                return True
            return ch.isupper()

        matching_sequences = [
            self._cleanse(sequence)
            for sequence in matching_sequences
            if sum([
                is_upper(gram[0])
                for gram in sequence.split()
            ]) == len(sequence.split())
        ]

        matching_sequences = [
            sequence
            for sequence in matching_sequences
            if not sequence.strip().startswith('.')
        ]

        return matching_sequences

    @staticmethod
    def _filter_matches(matches: List[str]) -> List[str]:
        normalized: List[str] = []

        def has_initial_start(match_tokens: List[str]) -> bool:

            # Prevent sequences like 'F. Halsey' from being included
            if len(match_tokens[0]) == 2 and match_tokens[0][-1] == '.':
                return True
            return False

        for match in matches:
            tokens = match.split()

            if has_initial_start(tokens):
                continue

            normalized.append(match)

        return normalized

    def process(self,
                sentence: Sentence) -> Dict[str, List[str]]:
        """
        Process the given sentence and extract people sequences using exact and fuzzy matching patterns.

        Args:
            sentence (Sentence): The input sentence to process.

        Returns:
            Dict[str, List[str]]: A dictionary containing the extracted people sequences.
                The keys are "exact" and "fuzzy", and the values are lists of strings representing the matches.
        """
        sw = Stopwatch()
        assert isinstance(sentence, Sentence)

        exact_matches: List[str] = self._extract_patterns(
            sentence=sentence, patterns=PATTERNS_1)

        fuzzy_matches: List[str] = self._extract_patterns(
            sentence=sentence, patterns=PATTERNS_2)

        exact_matches = self._filter_matches(
            sorted(set(exact_matches), reverse=True))

        fuzzy_matches = self._filter_matches(sorted(set([
            fuzzy_match for fuzzy_match in fuzzy_matches
            if fuzzy_match not in exact_matches
        ]), reverse=True))

        result = {
            "exact": exact_matches,
            "fuzzy": fuzzy_matches,
        }

        total_matches = len(exact_matches) + len(fuzzy_matches)

        if self.isEnabledForDebug and total_matches > 0:
            self.logger.debug(
                f"Extracted Topic Sequences: {total_matches} in ({str(sw)})")

        return result



# !/usr/bin/env python
# -*- coding: UTF-8 -*-


SYSTEM_PROMPT_SUMMARY = "You are a helpful assistant who condenses text and maintains tonality."


SYSTEM_PROMPT_TEMPLATE = """
You are a helpful assistant who summarizes text.

You must follow these rules:
1. You must only output the summary.  
2. Do not use phrases like "the author recalls" or "the text describes".  
3. Your summary must not sound like a summary.
4. Your summary must have the same tone as the original text.
5. Use phrases found in the original text for your summary.

CONTENT TO SUMMARIZE:
```
#SUMMARY
```
"""


SAMPLE_ORIGINAL_TEXT = """
inconsequential foes in any future war. Now many saw them as supermen who could appear anywhere at any time and in overwhelming force. Some alarmists predicted that they would target Hawaii or the Panama Canal, even the American West Coast. One particularly vulnerable American outpost was Samoa, exactly halfway between Hawaii and Australia (see Map 2). King told Nimitz that in addition to defending Hawaii, his primary task was to protect the line of communication to Australia. If the Japanese seized Samoa, as they had both Guam and Wake, it would sever that connection. In addition to its location, Samoa also hosted the best deepwater port in the South Pacific, at Pago Pago. Yet in January 1942, Samoa's garrison consisted of only about a hundred men of the 7th Marine Defense Battalion plus another hundred or so local militia known as the Fita Fita (Samoan for "soldier"). As if to demonstrate the garrison's vulnerability, an impertinent Japanese sub skipper surfaced off Samoa that month and shelled the island, though it turned out to be only a nuisance raid. To secure this invaluable outpost, King organized a convoy in San Diego to carry five thousand U.S. Marines there. Since the Yorktown was also in San Diego and bound for the Pacific, she and her escorts could accompany the convoy to protect it from Japanese interference.
The imminent arrival of the Yorktown in the Pacific was welcome news for Nimitz, and he challenged his staff to develop a scheme for the best use of this new resource, as well as the three other carrier task forces. Though he was never reluctant to make independent decisions, he always sought to obtain as much information as possible before doing so. In addition to Draemel and McMorris, therefore, he invited Pye to the morning conference. Pye had been deeply shocked by the Japanese attack on December 7, and that made him diffident now. He was also in the awkward position of being a senior vice admiral without a command.
Both Pye and Kimmel did their best to be helpful. Nimitz wrote Catherine that "everyone wants to help me, Pye and Kimmel most of all", but they were also sensitive about being in the way. At the morning meeting, Nimitz outlined the priorities that King had stipulated: defend Hawaii, protect the sea lanes to Australia, and suggested.
"""


SAMPLE_OUTPUT = """
Many viewed the Japanese as formidable adversaries, capable of striking anywhere with overwhelming force, causing concern about potential targets like Hawaii, the Panama Canal, and even the American West Coast. Samoa, positioned strategically between Hawaii and Australia, was identified as a vulnerable outpost crucial for communication. Its meager garrison of Marines and local militia underscored its vulnerability, highlighted by a minor Japanese raid. To reinforce Samoa, King arranged for a convoy carrying five thousand Marines from San Diego, coinciding with the departure of the Yorktown for the Pacific.
Nimitz eagerly anticipated the Yorktown's arrival, prompting strategic discussions with his staff, including Pye, who, despite his shock from the Pearl Harbor attack, contributed positively despite his lack of command. Nimitz emphasized King's priorities: defending Hawaii, safeguarding sea lanes to Australia, and proposing further actions.
"""


def generate_sample_prompt() -> str:
    return generate_prompt(input_text=SAMPLE_ORIGINAL_TEXT)


def generate_prompt(input_text: str) -> str:
    assert isinstance(input_text, str)
    return SYSTEM_PROMPT_TEMPLATE.replace('#SUMMARY', input_text)



from .token_parser_coordinates import TokenParserCoordinates
from .token_parser_normalize import TokenParserNormalize
from .token_parser_postprocess import TokenParserPostProcess
from .token_parser_punctuation import TokenParserPunctuation
from .token_parser_resultset import TokenParserResultSet
from .token_parser_spacy import TokenParserSpacy
from .token_parser_squots import TokenParserSquots
from .token_parser_wordnet import TokenParserWordnet




#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# pylint:disable=bad-whitespace
# pylint:disable=line-too-long
# pylint:disable=too-many-lines
# pylint:disable=invalid-name

# #########################################################
#
#       ************** !! WARNING !! ***************
#       ******* THIS FILE WAS AUTO-GENERATED *******
#       ********* DO NOT MODIFY THIS FILE **********
#
# #########################################################

wordnet_terms_j_provenance = {
    'action': ['router.py',
               'plac_core.py',
               'csv2py_orchestrator.py',
               'wordnet2py.py',
               'csv_list_writer.py',
               'common_utils.py'],
    'source': 'resources/data/csv/wordnet.csv',
    'time': '2022-05-17 17:00:36.172958'}

wordnet_terms_j = [
    'jab',
    'jabalpur',
    'jabbed',
    'jabber',
    'jabberer',
    'jabbering',
    'jabberwocky',
    'jabbing',
    'jabiru',
    'jabirus',
    'jaboncillo',
    'jabot',
    'jaboticaba',
    'jacamar',
    'jacamars',
    'jacaranda',
    'jacinth',
    'jack',
    'jackal',
    'jackals',
    'jackanapes',
    'jackass',
    'jackboot',
    'jackbooted',
    'jackdaw',
    'jacket',
    'jackets',
    'jackfruit',
    'jackhammer',
    'jackknife',
    'jacklight',
    'jackpot',
    'jackrabbit',
    'jacks',
    'jackscrew',
    'jacksmelt',
    'jacksnipe',
    'jackson',
    'jacksonia',
    'jacksonian',
    'jacksonville',
    'jackstones',
    'jackstraw',
    'jackstraws',
    'jacob',
    'jacobean',
    'jacobi',
    'jacobin',
    'jacobinic',
    'jacobinical',
    'jacobinism',
    'jacobins',
    'jacobite',
    'jacobs',
    'jacobus',
    'jaconet',
    'jacquard',
    'jacques',
    'jacquinia',
    'jactation',
    'jactitate',
    'jactitation',
    'jaculus',
    'jade',
    'jaded',
    'jadeite',
    'jadestone',
    'jaeger',
    'jaegers',
    'jafar',
    'jaffa',
    'jaffar',
    'jag',
    'jagannath',
    'jagannatha',
    'jagatai',
    'jagganath',
    'jaggary',
    'jagged',
    'jaggedly',
    'jaggedness',
    'jagger',
    'jaggery',
    'jagghery',
    'jaggy',
    'jaghatai',
    'jagua',
    'jaguar',
    'jaguarondi',
    'jaguarundi',
    'jahan',
    'jahvey',
    'jahweh',
    'jail',
    'jailbird',
    'jailbreak',
    'jailed',
    'jailer',
    'jailhouse',
    'jailor',
    'jain',
    'jainism',
    'jainist',
    'jak',
    'jakarta',
    'jakes',
    'jakob',
    'jakobson',
    'jalalabad',
    'jalapeno',
    'jalopy',
    'jalousie',
    'jam',
    'jamaica',
    'jamaican',
    'jamb',
    'jambalaya',
    'jambeau',
    'jamberry',
    'jambon',
    'jamboree',
    'jambos',
    'jambosa',
    'jambs',
    'james',
    'jamesian',
    'jameson',
    'jamesonia',
    'jamestown',
    'jamison',
    'jamjar',
    'jammed',
    'jammer',
    'jammies',
    'jamming',
    'jampack',
    'jampan',
    'jampot',
    'jams',
    'jan',
    'jane',
    'janeiro',
    'janet',
    'jangle',
    'jangling',
    'jangly',
    'janissary',
    'janitor',
    'jansen',
    'jansenism',
    'jansenist',
    'january',
    'janus',
    'jap',
    'japan',
    'japanese',
    'jape',
    'japery',
    'japheth',
    'japonica',
    'jar',
    'jarful',
    'jargon',
    'jargoon',
    'jarred',
    'jarrell',
    'jarring',
    'jars',
    'jasmine',
    'jasminum',
    'jason',
    'jasper',
    'jaspers',
    'jassid',
    'jassidae',
    'jat',
    'jati',
    'jatropha',
    'jaundice',
    'jaundiced',
    'jaunt',
    'jauntiness',
    'jaunty',
    'java',
    'javan',
    'javanese',
    'javanthropus',
    'javelin',
    'javelina',
    'jaw',
    'jawan',
    'jawbone',
    'jawbones',
    'jawbreaker',
    'jawed',
    'jawfish',
    'jawfishes',
    'jawless',
    'jaws',
    'jay',
    'jaybird',
    'jays',
    'jayshullah',
    'jaywalk',
    'jaywalker',
    'jazz',
    'jazzman',
    'jazzy',
    'jdam',
    'jealous',
    'jealously',
    'jealousy',
    'jean',
    'jeanne',
    'jeans',
    'jeddah',
    'jeep',
    'jeer',
    'jeered',
    'jeerer',
    'jeering',
    'jeers',
    'jeffers',
    'jefferson',
    'jeffersonian',
    'jehad',
    'jehovah',
    'jejune',
    'jejuneness',
    'jejunitis',
    'jejunity',
    'jejunoileitis',
    'jejunostomy',
    'jejunum',
    'jell',
    'jellaba',
    'jelled',
    'jellied',
    'jellies',
    'jellify',
    'jello',
    'jelly',
    'jellyfish',
    'jellyfishes',
    'jellyleaf',
    'jellylike',
    'jellyroll',
    'jem',
    'jemaah',
    'jemmy',
    'jena',
    'jenner',
    'jennet',
    'jennings',
    'jenny',
    'jensen',
    'jeopardise',
    'jeopardize',
    'jeopardy',
    'jerboa',
    'jerboas',
    'jeremiad',
    'jeremiah',
    'jeremy',
    'jerevan',
    'jerez',
    'jericho',
    'jerk',
    'jerked',
    'jerker',
    'jerkin',
    'jerkiness',
    'jerking',
    'jerks',
    'jerkwater',
    'jerky',
    'jernigan',
    'jeroboam',
    'jerome',
    'jerry',
    'jersey',
    'jerusalem',
    'jespersen',
    'jess',
    'jessamine',
    'jesse',
    'jessica',
    'jest',
    'jester',
    'jesting',
    'jesuit',
    'jesuitic',
    'jesuitical',
    'jesuitism',
    'jesuitry',
    'jesuits',
    'jesus',
    'jet',
    'jeth',
    'jetliner',
    'jets',
    'jetsam',
    'jetted',
    'jetting',
    'jettison',
    'jettisoned',
    'jetty',
    'jevons',
    'jew',
    'jewbush',
    'jewel',
    'jeweled',
    'jeweler',
    'jewelers',
    'jewelled',
    'jeweller',
    'jewellery',
    'jewelry',
    'jewels',
    'jewelweed',
    'jewess',
    'jewfish',
    'jewish',
    'jewison',
    'jewry',
    'jews',
    'jezebel',
    'jfk',
    'jhvh',
    'ji',
    'jiang',
    'jiao',
    'jib',
    'jibbed',
    'jibboom',
    'jibe',
    'jibs',
    'jidda',
    'jiddah',
    'jiffy',
    'jig',
    'jigger',
    'jiggered',
    'jiggermast',
    'jiggle',
    'jigsaw',
    'jihad',
    'jihadi',
    'jihadist',
    'jillion',
    'jilt',
    'jilted',
    'jilts',
    'jim',
    'jima',
    'jimdandy',
    'jimenez',
    'jimhickey',
    'jimmied',
    'jimmies',
    'jimmy',
    'jimsonweed',
    'jinghpaw',
    'jinghpo',
    'jingle',
    'jingling',
    'jingly',
    'jingo',
    'jingoism',
    'jingoist',
    'jingoistic',
    'jinja',
    'jinks',
    'jinnah',
    'jinnee',
    'jinni',
    'jinrikisha',
    'jinx',
    'jinxed',
    'jiqui',
    'jird',
    'jirga',
    'jirrbal',
    'jitney',
    'jitter',
    'jitterbug',
    'jitteriness',
    'jittering',
    'jitters',
    'jittery',
    'jiujitsu',
    'jive',
    'jnd',
    'jnr',
    'joachim',
    'joan',
    'joaquin',
    'job',
    'jobbed',
    'jobber',
    'jobbers',
    'jobbery',
    'jobcentre',
    'jobholder',
    'jobless',
    'jobs',
    'jocasta',
    'jock',
    'jockey',
    'jockstrap',
    'jocose',
    'jocoseness',
    'jocosity',
    'jocote',
    'jocular',
    'jocularity',
    'jocund',
    'jocundity',
    'jodhpur',
    'jodhpurs',
    'joe',
    'joel',
    'joewood',
    'joffre',
    'joffrey',
    'jog',
    'jogger',
    'joggers',
    'jogging',
    'joggle',
    'johannes',
    'johannesburg',
    'john',
    'johnny',
    'johnnycake',
    'johns',
    'johnson',
    'johnston',
    'johor',
    'johore',
    'join',
    'joined',
    'joiner',
    'joinery',
    'joining',
    'joins',
    'joint',
    'jointed',
    'jointer',
    'jointly',
    'joints',
    'jointure',
    'jointworm',
    'joist',
    'joists',
    'joke',
    'joker',
    'jokers',
    'jokes',
    'jokester',
    'joking',
    'joliet',
    'joliot',
    'jolliet',
    'jollification',
    'jollify',
    'jolliness',
    'jollity',
    'jolly',
    'jolson',
    'jolt',
    'jolted',
    'jolting',
    'jolts',
    'jolty',
    'jonah',
    'jonathan',
    'jones',
    'jonesboro',
    'jong',
    'jongleur',
    'jonquil',
    'jonson',
    'jook',
    'joplin',
    'joppa',
    'jordan',
    'jordanella',
    'jordanian',
    'jorum',
    'josef',
    'joseph',
    'josephus',
    'josh',
    'joshua',
    'josiah',
    'joss',
    'jostle',
    'jostled',
    'jostling',
    'josue',
    'jot',
    'jotted',
    'jotter',
    'jotting',
    'jotun',
    'jotunn',
    'joule',
    'joules',
    'jounce',
    'journal',
    'journalese',
    'journalism',
    'journalist',
    'journalistic',
    'journalists',
    'journals',
    'journey',
    'journeyer',
    'journeying',
    'journeyman',
    'journeys',
    'joust',
    'jousts',
    'jove',
    'jovial',
    'joviality',
    'jovian',
    'jowett',
    'jowl',
    'jowly',
    'joy',
    'joyce',
    'joyful',
    'joyfully',
    'joyfulness',
    'joyless',
    'joylessness',
    'joyous',
    'joyously',
    'joyousness',
    'joyride',
    'joyrides',
    'joys',
    'joystick',
    'jr',
    'jra',
    'juan',
    'juarez',
    'jubbulpore',
    'jubilance',
    'jubilancy',
    'jubilant',
    'jubilate',
    'jubilation',
    'jubilee',
    'juda',
    'judaea',
    'judah',
    'judaic',
    'judaica',
    'judaical',
    'judaism',
    'judas',
    'judder',
    'jude',
    'judea',
    'judge',
    'judged',
    'judgement',
    'judges',
    'judgeship',
    'judging',
    'judgment',
    'judgmental',
    'judgments',
    'judicable',
    'judicatory',
    'judicature',
    'judicial',
    'judicially',
    'judiciary',
    'judicious',
    'judiciousness',
    'judith',
    'judo',
    'jug',
    'jugale',
    'jugful',
    'juggernaut',
    'juggle',
    'juggled',
    'juggler',
    'jugglers',
    'jugglery',
    'juggles',
    'juggling',
    'juglandaceae',
    'juglandales',
    'juglans',
    'jugoslav',
    'jugoslavian',
    'jugoslavija',
    'jugs',
    'jugular',
    'juice',
    'juiceless',
    'juicer',
    'juices',
    'juiciest',
    'juiciness',
    'juicy',
    'jujitsu',
    'juju',
    'jujube',
    'jujubes',
    'jujutsu',
    'juke',
    'jukebox',
    'julep',
    'jules',
    'julia',
    'julian',
    'julienne',
    'julius',
    'july',
    'jumbal',
    'jumble',
    'jumbled',
    'jumbles',
    'jumbo',
    'jumbojet',
    'jument',
    'jumentous',
    'jump',
    'jumped',
    'jumper',
    'jumpers',
    'jumpiness',
    'jumping',
    'jumps',
    'jumpstart',
    'jumpsuit',
    'jumpy',
    'juncaceae',
    'juncaginaceae',
    'junco',
    'junction',
    'junctions',
    'juncture',
    'junctures',
    'juncus',
    'june',
    'juneau',
    'juneberry',
    'jung',
    'jungermanniaceae',
    'jungermanniales',
    'jungian',
    'jungle',
    'jungles',
    'jungly',
    'junin',
    'junior',
    'juniper',
    'junipers',
    'juniperus',
    'junk',
    'junker',
    'junkers',
    'junket',
    'junketeer',
    'junketing',
    'junkie',
    'junky',
    'junkyard',
    'juno',
    'junoesque',
    'junta',
    'junto',
    'jupati',
    'jupaty',
    'jupiter',
    'jural',
    'jurassic',
    'jure',
    'juridic',
    'juridical',
    'jurisdiction',
    'jurisdictional',
    'jurisdictions',
    'jurisprudence',
    'jurisprudential',
    'jurist',
    'juristic',
    'juror',
    'jurors',
    'jury',
    'juryman',
    'jurywoman',
    'jussieu',
    'just',
    'justice',
    'justiciar',
    'justiciary',
    'justifiable',
    'justifiably',
    'justification',
    'justificative',
    'justificatory',
    'justified',
    'justifier',
    'justifies',
    'justify',
    'justifying',
    'justinian',
    'justly',
    'justness',
    'jut',
    'jute',
    'jutes',
    'jutish',
    'jutland',
    'juts',
    'jutted',
    'jutting',
    'juvenal',
    'juvenescence',
    'juvenile',
    'juvenility',
    'juxtapose',
    'juxtaposed',
    'juxtaposes',
    'juxtaposing',
    'juxtaposition',
    'juxtapositions',
    'jv',
    'jyaistha',
    'jylland',
    'jynx']



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Add Wordnet Flag to Token """


from lingpatlab.baseblock import BaseObject


class TokenParserResultSet(BaseObject):
    """ Add Wordnet Flag to Token """

    def __init__(self):
        """ Change Log

        Created:
            13-Oct-2021
            craigtrim@gmail.com
            *   refactored out of 'parse-input-tokens' in pursuit of
                https://github.com/grafflr/graffl-core/issues/41
        Updated:
            14-Oct-2021
            craig.@graffl.ai
            *   retokenize spaces back into owning tokens
                https://github.com/grafflr/graffl-core/issues/48#issuecomment-943793697
            *   ensure unique id and head values even when text is duplicated
                https://github.com/grafflr/graffl-core/issues/53#issuecomment-944001757
        Updated:
            2-Sept-2022
            craigtrim@gmail.com
            *   fix defect in ID-to-HEAD relationship
                https://github.com/craigtrim/spacy-token-parser/issues/1#issuecomment-1236046400
        Updated:
            16-Sept-2022
            craigtrim@gmail.com
            *   rename component
                https://github.com/craigtrim/spacy-token-parser/issues/3
        Updated:
            14-Jul-2023
            craigtrim@gmail.com
            *   Ensure Unique Token ID and HEAD values
                https://bai.atlassian.net/browse/COR-134
        Updated:
            22-Mar-2024
            craigtrim@gmail.com
            *   fix spacing issue
                https://github.com/craigtrim/datapipe-apis/issues/64
        """
        BaseObject.__init__(self, __name__)

    def process(self,
                doc) -> list:
        """Transform spaCy doc into a result set

        Args:
            doc ([spacy]): a spaCy document

        Returns:
            list: a list of tokens
        """
        results = []

        i = 0

        tokens = [token for token in doc]
        while i < len(tokens):

            token = tokens[i]

            # ---------------------------------------------------------- ##
            # Purpose:    Reclaim Whitespace into Prior Tokens
            # Reference:  https://github.com/grafflr/graffl-core/issues/48#issuecomment-944003662
            # ---------------------------------------------------------- ##
            def has_trailing_space() -> bool:
                if i + 1 >= len(tokens):
                    return False
                return tokens[i + 1].text == ' '

            is_space_next = has_trailing_space()
            if is_space_next:
                i += 1

            def token_text() -> str:
                # ---------------------------------------------------------- ##
                # Purpose:    Fix Spacing
                # Reference:  https://github.com/craigtrim/datapipe-apis/issues/64
                # ---------------------------------------------------------- ##
                token_text: str = token.text
                if '\n' in token_text:
                    token_text = token_text.replace('\n', ' ')
                while '  ' in token_text:
                    token_text = token_text.replace('  ', ' ')
                if is_space_next and not token_text.endswith(' '):
                    return f'{token_text} '
                return token_text

            # ---------------------------------------------------------- ##
            # Purpose:      Ensure Unique IDs for duplicate text values
            # Reference:    https://github.com/grafflr/graffl-core/issues/53#issuecomment-944001757
            #
            # Updated:      Simplify ID and HEAD relationships
            #               https://github.com/craigtrim/spacy-token-parser/issues/1
            #
            # Updated:      Ensure Unique Token-ID and Token-HEAD Values
            #               https://bai.atlassian.net/browse/COR-134
            # ---------------------------------------------------------- ##
            token_id = f"{str(token.orth)}#{i}"
            token_head = f"{str(token.head.orth)}#{token.head.i}"

            d_morph = token.morph.to_dict()

            def tense() -> str:
                if 'Tense' in d_morph:
                    return d_morph['Tense'].lower()
                return ''

            def number() -> str:
                """ Plural vs Singular vs None """
                if 'Number' in d_morph:
                    n = d_morph['Number']
                    if n == 'Sing':
                        return 'singular'
                    elif n == 'Plur':
                        return 'plural'
                    raise NotImplementedError(n)  # I want to know!
                return ''

            def verb_form() -> str:
                """  """
                if 'VerbForm' in d_morph:
                    return d_morph['VerbForm'].lower()
                return ''

            results.append({
                'id': token_id,
                'text': token_text(),
                'lemma': token.lemma_,
                'tense': tense(),  # Past vs Future
                'noun_number': number(),  # Singular vs Plural
                'verb_form': verb_form(),  # Gerund vs Infinitive etc
                'sentiment': token.sentiment,
                'pos': token.pos_,
                'tag': token.tag_,
                'dep': token.dep_,
                'ent': token.ent_type_,
                'shape': token.shape_,
                'is_alpha': token.is_alpha,
                'is_stop': token.is_stop,
                'head': token_head,
                'other': {
                    'i': token.i,
                    'idx': token.idx,
                    'orth': token.orth,
                    'head_i': token.head.i,
                    'head_idx': token.head.idx,
                    'head_orth': token.head.orth,
                    'head_text': token.head.text
                },
            })

            i += 1

        return results
